#!/bin/sh
#
# Copyright (C) 2020 by The ACT Corp, all rights reserved.
#
#ident "@(#) orado.sh       1.5e   2020.09.30      Katsumi.Nuruki"
#
#
# variable name  description:
#
# SYSTEMPASSWORD This environment variable is usually "manager" that it is the
#                ORACLE system user account password. If it is different from
#                the target ORACLE system, it has to be changed.
# ORADO          It should be set as certain ORACLE system identifier. It will
#                be used for checking and setting $ORACLE_SID and $ORACLE_HOME
#                environment variables. These variables will determine
#                the ORACLE system files which are,
#                /etc/oratab and /etc/oraInst.loc(Linux) or
#		 /var/opt/oracle/oratab and /var/opt/oracle/oraInst.loc(SunOS).
#                In this schell script, $ORADO is used for as $SID.
# LOGDIR         This environment variable determines where STATSPACK data will
#                be gathered. $LOGDIR also use for the other orado.sh functions.
# HOSTNAME       This environment variable is used for the hostname identifier.
#                When the HOSTNAME environment variable isn't defined, it will
#                be assumed `uname -n` output of this system.
# PDBNAME        When using Oracle12c, set this environment variable, which
#                means the pluggable database name.
#
SYSTEMPASSWORD=manager
#
# Update information.
#
# orado.edit was created as first version.                   2003.01.06 Rev.1.0
# COR-001 sprepins.sql patch for edit.                       2003.01.06 Rev.1.0
# COR-002 spreport.sql patch for edit.                       2003.01.12 Rev.1.0
# COR-003 patch for Top 5 xxxx Events.                       2003.01.19 Rev.1.0
# COR-004 Add set LINESIZE 128 in sql.                       2004.05.11 Rev.1.0a
# COR-005 Add set LINESIZE 256 in sql.                       2004.05.12 Rev.1.0a
# COR-006 Add $HOSTNAME variable.                            2004.05.19 Rev.1.0b
# COR-007 Change SID & GID checking.                         2004.05.19 Rev.1.0b
# COR-008 Check version for Oracle10g.                       2004.08.26 Rev.1.1
# COR-009 sprepins.sql patch for edit.                       2004.08.27 Rev.1.1
# COR-010 Add for Oracle 10g new sections.                   2004.09.15 Rev.1.2
# COR-011 Script can be using for SunOS.                     2004.10.05 Rev.1.3
# COR-012 Avoid file deadlock in CYGWIN.                     2004.10.06 Rev.1.3
# COR-013 Add unpack description.                            2005.01.26 Rev.1.3a
# COR-014 Break if separator found.                          2006.02.28 Rev.1.3c
# COR-015 Add BufferwaitStatistics.                          2006.04.18 Rev.1.3c
# COR-016 Add getting SID for CYGWIN.                        2006.04.20 Rev.1.3c
# COR-017 Correct invalid function name.                     2006.04.23 Rev.1.3c
# COR-018 Correct report fail on CYGWIN(void).               2011.02.23 Rev.1.4f
# COR-019 Change for HP-UX.                                  2006.06.13 Rev.1.3e
# COR-020 Chg checking PATH for CYGWIN.                      2006.07.19 Rev.1.3f
# COR-021 Add $TOOLS variable.                               2006.07.19 Rev.1.3f
# COR-022 Add to move the exported file.                     2006.07.19 Rev.1.3f
# COR-023 Set NLS variables for default(void).               2011.02.23 Rev.1.4f
# COR-024 Change for Oracle 10g 10.2.                        2006.09.07 Rev.1.3g
# COR-025 Correct echo command.                              2007.07.15 Rev.1.3h
# COR-026 alter session nls parameters.                      2007.07.23 Rev.1.3h
# COR-027 Change for Oracle 11g.                             2010.02.14 Rev.1.4c
# COR-028 Change for sysdate or SYSDATE.                     2009.09.05 Rev.1.4a
# COR-029 Change for Oracle 11g R2 Wait Events.              2010.07.28 Rev.1.4d
# COR-030 Fix CheckAndSetOraEnv sid error on Linux.          2010.12.18 Rev.1.4e
# COR-031 Add proc on RAC environment(Windows).              2011.02.23 Rev.1.4f
# COR-032 LF into Global CURRNET & Cache Trf Stats.          2011.02.23 Rev.1.4f
# COR-033 Clr on.LST file after function finished.           2011.02.28 Rev.1.4f
# COR-034 Change date format.                                2011.03.01 Rev.1.4f
# COR-035 Add GlobalCRServedStats.                           2011.03.06 Rev.1.4f
# COR-036 Add GlobalEnqueueStats.                            2011.03.06 Rev.1.4f
# COR-037 Add Streams Pool Advisory in OEDSCR10.             2011.09.12 Rev.1.4g
# COR-038 Change formula.                                    2012.05.18 Rev.1.4h
# COR-039 Add -i option in the grep command.                 2012.05.18 Rev.1.4h
# COR-040 Change formula for Linux Oracle RAC.               2012.05.19 Rev.1.4h
# COR-041 Change for Solaris 11 echo cmd path.               2013.12.25 Rev.1.4i
# COR-042 Unset NLS_LANG variable.                           2013.12.29 Rev.1.4i
# COR-043 Change for Oracle 12c.                             2014.01.14 Rev.1.5
# COR-044 Add Buffer wait editting on Oracle 10g.            2014.01.20 Rev.1.5a
# COR-045 Add spdusr befor spcusr for Oracle 8i.             2014.01.21 Rev.1.5a
# COR-046 Correct spstart error.                             2014.01.22 Rev.1.5a
# COR-047 Check LEVEL for Oracle 12c.                        2014.01.24 Rev.1.5a
# COR-048 Change for Oracle 18c, and 19c.                    2020.09.20 Rev.1.5b
# COR-049 Change PDB connection syntax.                      2020.09.20 Rev.1.5b
# COR-050 For v$pdbs OPEN_MODE list.                         2020.09.20 Rev.1.5c
# COR-051 Change for Oracle 18c and 19c.                     2020.09.20 Rev.1.5d
# COR-052 Change for ORA-28014 error.                        2020.09.21 Rev.1.5d
# COR-053 Patch for spcusr.sql.                              2020.09.30 Rev.1.5e
# COR-054 Patch for spcpkg.sql.                              2020.09.30 Rev.1.5e
#

#
# FUNCTIONS: There are several functions of the orado.sh shell script.
#            Some functions need an argument 2, these are described under the
#            "arg2" field. In those, "[ ]" (square brackets) mean optional.
#
# function   arg2    description:
# clean              Clean up perfstat user account, tables, and procedures.
# export             Export the whole perfstat user.
#                    The export file will be storered under the directory,
#                            $LOGDIR/orastats_hostname_$SID
#                    The name of file will be:
#                           "spuexp.dmp.yymmddHHMMSS"
# import    [ifile]  Import the whole perfstat user data from the exported file.
# list               list contents of splistdir.
# param              Show ORACLE parameters that are necessary to run STATSPACK.
# purge     [nl-nh]  Purge snap(s). "nl" is losnapid. "nh" is hisnapid. If an
#                    argument 2 isn't set, "all" snaps will be purged.
# report             Make some spreports from the DATABASE into the splistdir.
# setup              Set up perfstat user account, tables, and procedures.
# start       min    Start an automated STATSPACK job by the spauto.sql script.
#                    This function needs the time interval value in minutes.
# status             Show perfstat information.
# stop               Stop the automated job which has already been scheduled.
# tar        [SID]   Archive the all files to the $LOGDIR directory.
#                    The name of file will be:
#                           "orastats_hostname_SID_yymmddHHMMSS.tar.gz"
#                    File fromat is tar archived and compressed.
# unpack     [SID]   Uncompress the file(s) from the $LOGDIR directory. #COR-013
#                    The files are "orastats_hostname_SID_yymmddHHMMSS.tar.gz",
#                    which will be gunzip and tar. If some files exist, orado.sh
#                    will show a list of files. You can chose the one of them.
#

# 
# set parameters.
SetParams () {
DEBUGMODE=NO
ONETIME=YES
SPREPORT=YES
UIDCHECK=YES
#

BATCHMODE=$BATCHMODE
LANG=C;export LANG
LISTDIR=splistdir
ONEDAYSECONDS=86400
PASSWDFILE=/etc/passwd
REPORTDIR=spreptdir
NUMBEROFJOBQ=10
#

YEARHEAD="20"	# COR-034

#
# COR-021 start.
#
if [ "${TOOLS}_x" = "_x" ]; then
	TOOLS=TOOLS
fi
tools=`$BIN/echo $TOOLS | $BIN/awk '{ printf("%s\n",tolower($1)) }'`
#
# COR-021 end.
#
# COR-023 start.
# COR-042 start.
unset NLS_LANG
export NLS_LANG
# COR-042 end.
# COR-023 end.
#
# COR-006 start.
if [ "${HOSTNAME}_x" = "_x" ]; then
	HOST=`$BIN/uname -n`
else
	HOST=$HOSTNAME
fi
#
# COR-006 end.
#
#
IMPFILELIST=impfile.list
#
# COR-011 start.
nawk=awk
case $OSNAME in
Linux)		ORATAB=/etc/oratab
		ORAINST=/etc/oraInst.loc		;;
SunOS)		nawk=nawk
		ORATAB=/var/opt/oracle/oratab
		ORAINST=/var/opt/oracle/oraInst.loc	;;
CYGWIN*)	UIDCHECK=NO				;;
# COR-019 start.
HP-UX)		ORATAB=/etc/oratab
		ORAINST=/var/opt/oracle/oraInst.loc	;;
# COR-019 end.
esac
# COR-011 end.
#
ORATMP=${LOGDIR}/orastats_${HOST}_${SID}
SELECTOR=$ORATMP/selectors
SQLDIR=$ORATMP/spsqlsdir
TEMPFILE=$ORATMP/ora.temp$$
TRACELOG=$ORATMP/orado.log
#
#
LEVEL102010=10.2.0.1.0								#COR-043
#
# COR-001 start.
# insert "set newpage 0;".
SEDSCR="
Buffer Pool Advisory for
Buffer wait Statistics for
Instance Recovery Stats for
Library Cache Activity for
PGA Aggr Target Histogram for
PGA Memory Advisory for
PGA Memory Stats for
SGA breakdown difference for
Undo Segment Stats for
"
# change Snaps format.
OEDSCR="
Buffer Pool Advisory for
PGA Memory Advisory for
Resource Limit Stats for
Shared Pool Advisory for
"
# shortened the ttitle headers.
CRBSEARCHTEXT="CR Blocks Served (RAC) per Segment for DB:"
CRBREPLACETEXT="CR Blks Srvd (RAC) per Seg for DB:"
CUBSEARCHTEXT="CU Blocks Served (RAC) per Segment for DB:"
CUBREPLACETEXT="CU Blks Srvd (RAC) per Seg for DB:"
#
# COR-001 end.

# COR-002 start.
# insert "set newpage 0;", for ORACLE level 8.1.7.
SPRSCR="
Buffer wait Statistics for
Library Cache Activity for
SGA breakdown difference for
"
#
# COR-002 end.

# COR-003 start.
# Change the header format of the Top 5 Wait Events.
TOP5SCR="
Top 5 Wait Events
Top 5 Timed Events
"
#
# COR-003 end.

# COR-027 start.
# COR-032 set newpage 0 into Global CURRENT Served and Global Cache Transfer Stats.
# insert "set newpage 0;".
SEDSCR11="
ttitle lef 'Background Wait Events  '
ttitle lef 'Buffer Pool Advisory  '
ttitle lef 'Buffer Pool Statistics  '
ttitle lef 'Buffer wait Statistics  '
ttitle lef 'Global CURRENT Served Stats  '
ttitle lef 'Global Cache Transfer Stats  '
ttitle lef 'IO Stat by Function - detail  '
ttitle lef 'Instance Recovery Stats  '
ttitle lef 'Java Pool Advisory  '
ttitle lef 'Library Cache Activity  '
ttitle lef 'Memory Dynamic Components  '
ttitle lef 'Memory Resize Operations  '
ttitle lef 'OS Statistics  '
ttitle lef 'OS Statistics - detail  '
ttitle lef 'PGA Aggr Target Histogram  '
ttitle lef 'PGA Memory Advisory  '
ttitle lef 'Resource Limit Stats  '
ttitle lef 'Time Model System Stats  '
ttitle lef 'SGA breakdown difference  '
ttitle lef 'Wait Events (fg and bg) '
"
# COR-027 end.

# COR-009 start.
# COR-032 set newpage 0 into Global CURRENT Served and Global Cache Transfer Stats.
# COR-044 set newpage 0 into Buffer wait Statistics.
# insert "set newpage 0;".
SEDSCR10="
ttitle lef 'Buffer Pool Advisory  '
ttitle lef 'Buffer wait Statistics  '
ttitle lef 'Global CURRENT Served Stats  '
ttitle lef 'Global Cache Transfer Stats  '
ttitle lef 'Instance Recovery Stats  '
ttitle lef 'Java Pool Advisory  '
ttitle lef 'Library Cache Activity  '
ttitle lef 'OS Statistics  '
ttitle lef 'PGA Aggr Target Histogram  '
ttitle lef 'PGA Memory Advisory  '
ttitle lef 'Time Model System Stats  '
ttitle lef 'SGA breakdown difference  '
"
# change Snaps format.
OEDSCR10="
ttitle lef 'Buffer Pool Advisory  '
ttitle lef 'Java Pool Advisory  '
ttitle lef 'PGA Memory Advisory  '
ttitle lef 'Resource Limit Stats  '
ttitle lef 'Shared Pool Advisory  '
ttitle lef 'Streams Pool Advisory  '
"
# COR-037

# Change the header format of the Top 5 Wait Events.
TOP5SCR="
Top 5 Wait Events
Top 5 Timed Events
"
#
# COR-009 end.
}

#
# Log records and display.
#
LogRecord () {
arg1="$1"
function=`$BIN/echo $FUNCTION |
$BIN/$nawk '{ printf("%-6s\n",substr(toupper($1),1,6)) }'`		#COR-011
DATE=`$BIN/date '+%y%m%d%H%M%S'`
if [ -d $ORATMP ]; then
	$BIN/echo "$DATE $function :`$BIN/echo "$1"`">> $TRACELOG
fi
if [ "${BATCHMODE}_x" = "_x" ]; then
	$BIN/echo "$arg1"
else
	if [ $BATCHMODE = "NO" ]; then
		$BIN/echo "$arg1"
	fi
fi
}

#
# Clear work files and exit.
# COR-033 clear on.LST file in the execdirectory.
ClearfAndExit () {
WORKFILELIST="
$TEMPFILE
$SPOOLFILE
$ORATMP/ora.temp$$
$ORATMP/$SPOOLFILE
$ORATMP/on.lst
$ADMINDIR/ora.temp$$
$ADMINDIR/$SPOOLFILE
$EXECDIRECTORY/ora.temp$$
$EXECDIRECTORY/$SPOOLFILE
$EXECDIRECTORY/on.LST
$EXECDIRECTORY/on.lst
$WORKFILE
"
RETVAL=$1
if [ "${RETVAL}_x" = "_x" ]; then
	RETVAL=0
fi 
if [ $DEBUGMODE = "NO" ]; then
	for i in `$BIN/echo "$WORKFILELIST"`
	do
		if [ -s $i ]; then
			$BIN/rm $i
		fi
	done
fi
exit $RETVAL
}

#
# set system password.
#
ChgSysPasswd () {
input=NO
retrycount=3
N=1
while [ $input = "NO" ]
do
	$BIN/stty echo -echo
	$ECHOCMD -n "         ---enter system password : "
	read INPUT
	blankline
	if [ "${INPUT}_x" != "_x" ]; then
		$ECHOCMD -n "         re-enter system password : "
		read RINPUT
		if [ "$INPUT" = "$RINPUT" ]; then
			input=YES
			break
		else
			if [ $N -ge $retrycount ]; then
				blankline
				$BIN/stty echo echo
				LogRecord "ERROR: password entering count exhausted."
				ClearfAndExit 1
			fi
		fi
		N=`$USRBIN/expr $N + 1`
	fi
	blankline
done
$BIN/stty echo echo
$BIN/ed -s $ORADOEXECSHELL 1>$TEMPFILE 2>&1 <<-EOF
/SYSTEMPASSWORD=/
d
.-1
a
SYSTEMPASSWORD=${INPUT}
.
w $ORADOEXECSHELL
q
EOF
if [ $? -ne 0 ]; then
	LogRecord "ERROR: system password hasn't been set."
	ClearfAndExit 1
fi
blankline
}

#
# get the command path instead of which(1) command.
#
whichcmd () {
mod=$1
if [ "${mod}_x" = "_x" ]; then
	LogRecord "ERROR: whichcmd argument missing."
	ClearfAndExit 1
fi
COMMANDPATH=NOTFOUND
found=NO
while [ $found = "NO" ]
do
	for i in `$BIN/echo $PATH | $BIN/awk '{
		n = split($0,array,":")
		for ( i = 1 ; i <= n ; i++ )
			printf("%s\n",array[i])
		printf("\n") }'`
	do
		if [ -x ${i}/${mod} ]; then
			COMMANDPATH=${i}/${mod}
			found=YES
			break
		fi
	done
	found=$COMMANDPATH
done
}

#
# Convert the time string to the integer in seconds.
#
S2I() {
TIMEVALUE=`$BIN/echo $1 |
$BIN/awk '{ n=split($1,array,":") }
END {	if ( n > 1 ) {
		if (( array[1] < 24 ) && ( array[1] >= 0 ))
			hsec=array[1]*3600
		else {
			if ( array[1] == 24 ) {
				printf("%d\n",array[1]*3600)
				exit
			}
			printf("INVALID\n")
			exit
		}
		if (( array[2] < 60 ) && ( array[2] >= 0 ))
			msec=array[2]*60
		else {
			printf("INVALID\n")
			exit
		}
		if ( n == 3 ) {
			if (( array[3] < 60 ) && ( array[3] >= 0 ))
				sec=array[3]
			else {
				printf("INVALID\n")
				exit
			}
		}
		else
			sec=0
		printf("%d\n",hsec+msec+sec)
		}
	else
		printf("INVALID\n") }'`
if [ $TIMEVALUE = "INVALID" ]; then
	LogRecord "ERROR: the time format invalid."
	ClearfAndExit 1
fi
}

#
# Convert the integer second to the time string.
#
I2S() {
TIMESTRING=`$BIN/echo $1 |
$BIN/awk '{	h=$1/3600
		r=$1%3600
		m=r/60
		s=r%60
		if ( h >= 24 )
			h=h-24
		printf("%02d:%02d:%02d\n",h,m,s) }'`
}

#
# Check and Set ORACLE environment variables.
#
#
CheckAndSetOraEnv () {
sid=$1
if [ $OS = "Linux" ]; then
	# COR-007 start.
	oracle_sid=`
	$BIN/awk -F: '{
	if (( $1 == substr("'"$sid"'",1,length($1))) && (( substr($3,1,1) == "Y" ) || ( substr($3,1,1) == "N"))) {	# COR-030 COR-040
		sid="'"$sid"'"
		found="YES"
		exit
		}
	}
	END {	if ( found == "YES" )
			printf("%s\n",sid)
		else
			printf("NO\n")
	}' $ORATAB`
	if [ $oracle_sid = "NO" ]; then
		LogRecord "INFO:  the system identifier $SID hasn't been set."
		# ClearfAndExit 1
	fi
	ORACLE_SID=$sid
	oracle_home=`
	$BIN/awk -F: '{
	if (( $1 == substr("'"$ORACLE_SID"'",1,length($1))) && (( substr($3,1,1) == "Y" ) || ( substr($3,1,1) == "N"))) {	# COR-040
		oraclehome=$2
		found="YES"
		exit
		}
	}
	END {
	if ( found == "YES" )
		printf("%s\n",oraclehome)
	else
		printf("NO\n")
	}' $ORATAB`
	if [ $oracle_home = "NO" ]; then
		LogRecord "ERROR: an ORACLE home directory couldn't be found."
		# ClearfAndExit 1
	# COR-040 start.
	else
		ORACLE_HOME=$oracle_home
	# COR-040 end.
	fi
else
	ORACLE_SID=$sid
	# COR-007 end.
fi
export ORACLE_SID
export ORACLE_HOME
}

#
# Check the pmon process exists.
#
CheckPmon () {
SID=$1
if [ $# -eq 2 ]; then
	RETURN=$2
	if [ $RETURN = "RETURN" ]; then
		RETURN=YES
	else
		RETURN=NO
	fi
else
	RETURN=NO
fi
if [ $OS = "Linux" ] || [ $OS = "SunOS" ] || [ $OS = "HP-UX" ]; then	#COR-019
	pname="ora_pmon_${SID}"
	pmon=`
	$BIN/ps -efl | $BIN/grep -v grep |
	$BIN/awk '{	if ( $NF == "'"$pname"'" ) {
				found="YES"
				exit
				}
			}
		END {	if ( found == "YES" )
				printf("OK\n")
			else
				printf("NG\n") }'`
else
	# for cygwin environment.
	pmon=`
	$BIN/ps -W |
	$BIN/grep -i ORACLE.EXE > /dev/null	# COR-039
	if [ $? -eq 0 ]; then			# COR-038
		$BIN/echo "OK"
	else
		$BIN/echo "NG"
	fi`
fi
if [ "${pmon}" = "NG" ]; then
	if [ $RETURN != "YES" ]; then
		LogRecord "ERROR: database ${ORACLE_SID} hasn't been started."
		ClearfAndExit 1
	fi
	LogRecord "INFO:  database ${ORACLE_SID} hasn't been started."
	RETVAL=NG
else
	RETVAL=OK
fi
}

#
# Check sqlcode in the spool file.
# If an error was found, the script will exit with return value 1.
#
CheckSQLcode () {
SQLCODE=`$BIN/awk '{	if ( $1 == "sqlcode" ) {
				getit="YES"
				retval=$2
				exit
				}
			}
END {	if ( getit == "YES" )
		printf("%d\n",retval)
	else
		printf("99999\n") }' $SPOOLFILE`
if [ $SQLCODE -ne 0 ]; then
	LogRecord "ERROR: ORA-`$BIN/echo ${SQLCODE} | $BIN/awk '{ printf("%05d\n",$1) }'` has occurred. please check log files."
	ClearfAndExit 1
fi
}

#
# Set the times environment variables.
#
SetTimeParameters() {
arg1=$1
arg2=$2
arg3=$3
#
# $arg1         : sampling interval in minutes.
# $arg2         : start time now or HH:MM[:SS] format.
# $arg3         : end time xxmin, xxhour, or HH:MM[:SS] format.
#
# This function sets times environment variables as below.
# $SAMPLING     : sampling mode. INTHEDAY   = $arg2 < $arg3
#                                OVERTHEDAY = $arg2 > $arg3
# $INTERVAL	: sampling interval in minutes.
# $COUNT	: sampling count.
# $ctime        : current time in seconds.
# $CTIME        : current time with HH:MM:SS format.
# $stime	: start time in seconds.
# $STIME	: start time with HH:MM:SS format.
# $etime	: end time in seconds.
# $ETIME	: end time with HH:MM:SS format.
#
INTERVAL=`$BIN/echo $arg1 |
$BIN/awk '{
if (( $1 <= 60 ) && ( $1 >= 1 ))
		n=$1
	else
		n="NG"
	}
END { printf("%d\n",n) }'`
if [ $INTERVAL = "NG" ]; then
	LogRecord "ERROR: the sampling interval invalid."
	ClearfAndExit 1
fi
CTIME=`$BIN/date '+%H:%M:%S'`
S2I $CTIME
ctime=$TIMEVALUE
if [ $arg2 = "now" ]; then
	STIME=$CTIME
else
	STIME=$arg2
fi
S2I $STIME
stime=$TIMEVALUE
$BIN/echo $arg3 | $BIN/grep ":" 1>/dev/null 2>&1
if [ $? -eq 0 ]; then
	S2I $arg3
	etime=$TIMEVALUE
else
	RETVAL=`$BIN/echo $arg3 |
	$BIN/awk '{
	if ( substr($1,length($1)-3+1,3) == "min" ) {
		split($1,array,"min")
		if (( array[1] < 1440 ) && ( array[1] >= 1 ))
			n=array[1]*60
		else
			n="NG"
		}
	else
		if ( substr($1,length($1)-4+1,4 ) == "hour" ) {
			split($1,array,"hour")
			if (( array[1] <= 24 ) && ( array[1] >= 1 ))
				n=array[1]*3600
			else
				n="NG"
			}
		else
			n=$1
		}
	END { printf("%s\n",n) }'`
	if [ $RETVAL = "NG" ]; then
		LogRecord "ERROR: the end parameter is invalid."
		ClearfAndExit 1
	fi
	etime=`$USRBIN/expr $RETVAL + $stime`
fi
TIMEDIFF=`$USRBIN/expr $etime - $stime`
if [ $TIMEDIFF -eq $ONEDAYSECONDS ]; then
	interval=`$USRBIN/expr $INTERVAL \* 60`
	etime=`$USRBIN/expr $etime - $interval`
fi
I2S $etime
ETIME=$TIMESTRING
if [ $stime -eq $etime ]; then
	LogRecord "ERROR: the start and the end time are identical."
	ClearfAndExit 1
fi
if [ $stime -gt $etime ]; then
	etime=`$BIN/echo $etime |
	$BIN/awk '{ printf("%d\n",'"$ONEDAYSECONDS"' + $1) }'`
	SAMPLING=OVERTHEDAY
else
	if [ $etime -gt $ONEDAYSECONDS ]; then
		SAMPLING=OVERTHEDAY
	else
		SAMPLING=INTHEDAY
	fi
fi
COUNT=`$BIN/echo $INTERVAL |
$BIN/awk '{ printf("%d\n",(('"$etime"'-'"$stime"')/60)/$1) }'`
if [ $COUNT -eq 0 ]; then
	LogRecord "ERROR: the time parameters are inconsistent."
	ClearfAndExit 1
fi
}

#
# Print "."(dot) without LF.
#
printdot () {
if [ "${BATCHMODE}_x" = "_x" ]; then
	$ECHOCMD -n "."
else
	if [ $BATCHMODE = "NO" ]; then
		$ECHOCMD -n "."
	fi
fi
}

#
# Feed one line.
#
blankline () {
if [ "${BATCHMODE}_x" = "_x" ]; then
	$BIN/echo ""
else
	if [ $BATCHMODE = "NO" ]; then
		$BIN/echo ""
	fi
fi
}

#
# Check system user account password.
#
CheckSysPasswd () {
$ORABIN/sqlplus system/$SYSTEMPASSWORD 1>$TEMPFILE 2>&1 <<-EOF
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
spool off
EOF
if [ $? -ne 0 ]; then		# COR-038
	LogRecord "ERROR: system password could be incorrect."
	ClearfAndExit 1
fi
}
#

#
# Check perfstat user account.
#
CheckUserAccount () {
RETURN=$1
if [ "${RETURN}_x" = "_x" ]; then
	RETURN="NO"
fi
CheckSysPasswd
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	username
	from dba_users
	where username = 'PERFSTAT';
	show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
USERACTIVE=`$BIN/awk '{
	if ( $1 == "PERFSTAT" ) {
		found="YES"
		exit
		}
	}
END {	if ( found == "YES" )
		printf("%s\n",found)
	else
		printf("NO\n")
	}' $SPOOLFILE`
if [ $USERACTIVE = "NO" ]; then
	if [ $RETURN != "RETURN" ]; then
		LogRecord "ERROR: perfstat user isn't created."
		ClearfAndExit 1
	fi
fi
}

#
# Get JOB number.
#
GetJobNumber () {
RETURN=$1
if [ "${RETURN}_x" = "_x" ]; then
	RETURN="NO"
fi
CheckPmon $SID
CheckUserAccount RETURN
if [ $USERACTIVE = "YES" ]; then
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select job from user_jobs;
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
	JOBNO=`$BIN/awk '{
		if ( $1 == "JOB" ) {
			found="YES"
			getline
			getline
			jobno=$1
			}
		}
	END {	if ( found == "YES" )
			printf("%s\n",jobno)
		else
			printf("NO\n")
	}' $SPOOLFILE`
else
	if [ $RETURN != "RETURN" ]; then
		LogRecord "ERROR: perfstat user account hasn't been created."
		ClearfAndExit 1
	fi
	JOBNO="NO"
fi
}

#
# Convert the date string to the numeric string.
#
DateConv () {
if [ "${1}_x" != "_x" ]; then
	DATESTRING=`$BIN/echo $1 |
	$BIN/awk '{
		split($1,array,"-")
		day=array[1]
		year=sprintf("20%02d",array[3])
		if ( array[2] == "Jan" )
			month=1
		if ( array[2] == "Feb" )
			month=2
		if ( array[2] == "Mar" )
			month=3
		if ( array[2] == "Apr" )
			month=4
		if ( array[2] == "May" )
			month=5
		if ( array[2] == "Jun" )
			month=6
		if ( array[2] == "Jul" )
			month=7
		if ( array[2] == "Aug" )
			month=8
		if ( array[2] == "Sep" )
			month=9
		if ( array[2] == "Oct" )
			month=10
		if ( array[2] == "Nov" )
			month=11
		if ( array[2] == "Dec" )
			month=12
		exit
	}
	END { printf("%s.%02d.%02d\n",year,month,day) }'`
else
	DATESTRING=NULL
fi
}

#
# Get NEXT_SEC from user_jobs.
#
GetNextSec () {
CheckPmon $SID
CheckUserAccount RETURN
if [ $USERACTIVE = "YES" ]; then
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select next_sec from user_jobs;
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
	NEXT_SEC=`$BIN/awk '{
		if ( $1 == "NEXT_SEC" ) {
			found="YES"
			getline
			getline
			nextsec=$1
			}
		}
	END {	if ( found == "YES" )
			printf("%s\n",nextsec)
		else
			printf("NO\n")
	}' $SPOOLFILE`
fi
}

#
# COR-043 start.
SetCONNstring () {
TAG=$1
case $TAG in
perfstat) CONN=perfstat/perfstat      ;;
system)   CONN=system/$SYSTEMPASSWORD ;;
esac
if [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	# COR-048 COR-051
	CONN=${CONN}@localhost/${PDBNAME}				# COR-049
fi
}
# COR-043 end.
#
# execute spreport.sql so that it makes sp*lst files from the DB.
#
DoSpReportSQL () {
if [ -d $ORATMP/$LISTDIR ]; then
	$BIN/ls $ORATMP/$LISTDIR/sp_*_*.lst 2>/dev/null |
	$BIN/grep "sp_" | $BIN/grep ".lst" 1>/dev/null 2>&1
	if [ $? -eq 0 ]; then
		SPFILES=FOUND
	else
		SPFILES=NO
	fi
	if [ $SPFILES = "FOUND" ]; then
		LINES=`$USRBIN/wc -l $ORATMP/$LISTDIR/sp_*_*.lst |
		$BIN/awk '{	lines=$1
				if ( $2 == "total" ) {
					total="YES"
					exit
					}
				else
					n=NR
				}
		END {	if (( total == "YES" ) || ( n == 1 ))
				printf("%d\n",lines)
			else
				printf("NO\n") }'`
		if [ $LINES != "NO" ]; then
			if [ ${LINES} -gt 0 ]; then
				$BIN/rm $ORATMP/$LISTDIR/sp_*_*.lst
			fi
		fi
	fi
else
	$BIN/mkdir $ORATMP/$LISTDIR
fi
cd $ORATMP/$LISTDIR
if [ -s $ADMINDIR/sprepins.sql ]; then
	$BIN/cp $ADMINDIR/sprepins.sql sprepins.sql
	if [ $? -ne 0 ]; then		# COR-038
		LogRecord "ERROR: sprepins.sql copy failed."
		ClearfAndExit 1
	fi
	# COR-008 start.
	if [ -s $ADMINDIR/sprepcon.sql ] && [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-027 COR-043 COR-051
		$BIN/cp $ADMINDIR/sprepcon.sql sprepcon.sql
		if [ $? -ne 0 ]; then		# COR-038
			LogRecord "ERROR: sprepcon.sql copy failed."
			ClearfAndExit 1
		fi
	fi
	# COR-008 end.
	# COR-001 start.
	#
	# Patch to the sprepins.sql script.
	#
	##
	## insert "set newpage 0;" before the line of the ttitles.
	##
	# COR-009 start.
	if [ $LEVEL = "10" ]; then
		N=`$BIN/echo "$SEDSCR10" |
		$USRBIN/wc -l |
		$BIN/awk '{ printf("%d\n",$1+1) }'`
	else
		# COR-027 start.
		if [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then			#COR-043 COR-051
			N=`$BIN/echo "$SEDSCR11" |
			$USRBIN/wc -l |
			$BIN/awk '{ printf("%d\n",$1+1) }'`
		else 
		# COR-027 end.
			N=`$BIN/echo "$SEDSCR" |
			$USRBIN/wc -l |
			$BIN/awk '{ printf("%d\n",$1+1) }'`
		fi	# COR-027
	fi
	# COR-009 end.
	I=1
	while [ $I -lt $N ]
	do
		# COR-009 start.
		if [ $LEVEL = "10" ]; then
			TEXT=`$BIN/echo "$SEDSCR10" |
			$BIN/awk 'BEGIN { i = '"$I"' }
			{	if ( i == NR )
					printf("%s\n",$0)
				}'`
		else
			# COR-027 start.
			if [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then		#COR-043 COR-051
				TEXT=`$BIN/echo "$SEDSCR11" |
				$BIN/awk 'BEGIN { i = '"$I"' }
				{       if ( i == NR )
						printf("%s\n",$0)
				}'`
			else
			# COR-027 end.
				TEXT=`$BIN/echo "$SEDSCR" |
				$BIN/awk 'BEGIN { i = '"$I"' }
				{	if ( i == NR )
						printf("%s\n",$0)
					}'`
			fi	# COR-027
		fi
		# COR-009 end.
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$TEXT/
		.-1
		a
		set newpage 0;
		.
		w sprepins.sql
		q
		EOF
		# COR-029 start.
		WAITEVENTS=`$BIN/echo "$TEXT" |
		$BIN/awk '{	if (( substr($3,2,4) == "Wait" ) && ( $4 == "Events" ) && ( $6 == "and" ))
					flag="TRUE"
				else
					flag="FALSE"
				} END {
				printf("%s\n",flag)
		}'`
		if [ $WAITEVENTS = "TRUE" ]; then
			$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
			/$TEXT/
			s/Wait Events (fg and bg)/Wait Events/g
			w sprepins.sql
			q
			EOF
		fi
		# COR-029 end.
		I=`$USRBIN/expr $I + 1`
	done
	##
	## change Snaps format.
	##
	# COR-009 start.
	if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then         #COR-027 COR-043 COR-051
		N=`$BIN/echo "$OEDSCR10" |
		$USRBIN/wc -l |
		$BIN/awk '{ printf("%d\n",$1+1) }'`
	else
		N=`$BIN/echo "$OEDSCR" |
		$USRBIN/wc -l |
		$BIN/awk '{ printf("%d\n",$1+1) }'`
	fi
	# COR-009 end.
	I=1
	while [ $I -lt $N ]
	do
		# COR-009 start.
		if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then		#COR-027 COR-043 COR-051
			TEXT=`$BIN/echo "$OEDSCR10" |
			$BIN/awk 'BEGIN { i = '"$I"' }
			{	if ( i == NR )
					printf("%s\n",$0)
				}'`
			TTEXT="'Snaps: ' format 999999 begin_snap '-' format 999999 end_snap -"
		else
			TEXT=`$BIN/echo "$OEDSCR" |
			$BIN/awk 'BEGIN { i = '"$I"' }
			{	if ( i == NR )
					printf("%s\n",$0)
				}'`
			TTEXT="'Snaps: ' format 999999 begin_snap ' -' format 999999 end_snap -"
		fi
		# COR-009 end.
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$TEXT/
		.+2
		d
		.-1
		a
	           $TTEXT
		.
		w sprepins.sql
		q
		EOF
		I=`$USRBIN/expr $I + 1`
	done
	# COR-027 start.
	if [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ];then				#COR-043 COR-051
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/ttitle lef 'Undo Segment Stats  '-/
		.-1
		a
		set newpage 0;
		.
		/ttitle lef 'Top Process Memory (by component)  '-/
		d
		.
		.-1
		a
		set newpage 0;
		ttitle lef 'Top Process Memory by component '-
		.
		/ttitle lef 'Undo Segment Summary  '-/
		.-1
		a
		set newpage 0;
		.
		/Performance Summary continued/
		.+2
		d
		d
		.+1
		a
		set newpage 0;
		ttitle lef 'Performance Summary '-
		'DB/Inst: ' db_name '/' inst_name '  '-
		'Snaps: ' format 99999999 begin_snap '-' format 99999999 end_snap -
		skip 1;
		.
		/SQL ordered by Elapsed time for/
		d
		d
		d
		.-1
		a
		set newpage 0;
		ttitle lef 'SQL ordered by Elapsed time ' -
		'DB/Inst: ' db_name '/' inst_name '  '-
		'Snaps: ' format 99999999 begin_snap '-' format 99999999 end_snap -
		.
		/select 'Note: There is a '/
		.-2
		d
		.
		/ttitle lef 'SGA Target Advisory  '-/
		.+2
		d
		.-1
		a
		'Snaps: ' format 99999999 begin_snap '-' format 99999999 end_snap -
		.
		w sprepins.sql
		q
		EOF
	fi
	# COR-027 end.
	# COR-024 start.
	if [ $LEVEL = "10" ] && [ $SUBLEVEL = "2" ];then
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/ttitle lef 'Undo Segment Stats  '-/
		.-1
		a
		set newpage 0;
		.
		/ttitle lef 'Top Process Memory (by component)  '-/
		d
		.
		.-1
		a
		set newpage 0;
		ttitle lef 'Top Process Memory by component '-
		.
		/ttitle lef 'Undo Segment Summary  '-/
		.-1
		a
		set newpage 0;
		.
		/Performance Summary continued/
		.+2
		d
		d
		.+1
		a
		set newpage 0;
		ttitle lef 'Performance Summary '-
		'DB/Inst: ' db_name '/' inst_name '  '-
		'Snaps: ' format 99999999 begin_snap '-' format 99999999 end_snap -
		skip 1;
		.
		/select 'Note: There is a '/
		.-2
		d
		.
		/ttitle lef 'SGA Target Advisory  '-/
		.+2
		d
		.-1
		a
		'Snaps: ' format 99999999 begin_snap '-' format 99999999 end_snap -
		.
		w sprepins.sql
		q
		EOF
	fi
	# COR-024 end.
	if [ $LEVEL != "10" ] && [ $LEVEL != "11" ] && [ $LEVEL != "12" ] && [ $LEVEL != "18" ] && [ $LEVEL != "19" ]; then      #COR-009 COR-027 COR-043 COR-051
		##
		## shorten the ttitles headers.
		##
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$CRBSEARCHTEXT/
		s/$CRBSEARCHTEXT/$CRBREPLACETEXT/
		w sprepins.sql
		q
		EOF
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$CUBSEARCHTEXT/
		s/$CUBSEARCHTEXT/$CUBREPLACETEXT/
		w sprepins.sql
		q
		EOF
		# COR-009 start.
	else
		#
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/Instance Activity Statistics - absolute values/
		.+4
		s/Instance Activity Stats/Instance Activity Stats Values/
		.-1
		a
		set newpage 0;
		.
		w sprepins.sql
		q
		EOF
		#
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/Non-sysstat Instance Activity Statistics/
		.+4
		s/Instance Activity Stats/Instance Activity Stats Derived/
		.-1
		a
		set newpage 0;
		.
		w sprepins.sql
		q
		EOF
		#
		# COR-009 end.
	fi
	#
	# COR-001 end.
	# COR-003 start.
	# Change for Top 5 Wait Events.
	#
	N=`$BIN/echo "$TOP5SCR" |
	$USRBIN/wc -l |
	$BIN/awk '{ printf("%d\n",$1+1) }'`
	I=1
	while [ $I -lt $N ]
	do
		# COR-009 start.
		if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then		#COR-027 COR-043 COR-051
			N=1
			TEXT="Top &&top_n_events Timed Events"
			TTEXT="ttitle lef 'Top N Timed Events ' -"
			RTEXT="'DB/Inst: ' db_name '/' inst_name '  '-"
			STEXT="'Snaps: ' format 999999 begin_snap '-' format 999999 end_snap -"
		else
			TEXT=`$BIN/echo "$TOP5SCR" |
			$BIN/awk 'BEGIN { i = '"$I"' }
			{	if ( i == NR )
					printf("%s\n",$0)
				}'`
			TTEXT="ttitle lef '$TEXT for ' -"
			RTEXT="'DB: ' db_name  '  Instance: ' inst_name '  '-"
			STEXT="'Snaps: ' format 999999 begin_snap ' -' format 999999 end_snap -"
		fi
		# COR-009 end.
		$BIN/ed -s sprepins.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$TEXT/
		.-2
		a
		set newpage 0
		$TTEXT
	       		   $RTEXT
			   $STEXT
	       		skip 2;
		.
		w sprepins.sql
		q
		EOF
		I=`$USRBIN/expr $I + 1`
	done
	# COR-003 end.
fi
$BIN/cp $ADMINDIR/spreport.sql spreport.sql
if [ $? -ne 0 ]; then		# COR-038
	LogRecord "ERROR: spreport.sql copy failed."
	ClearfAndExit 1							#COR-017
fi
if [ $LEVEL = "8" ]; then						#COR-008
	#
	# COR-002 start.
	#
	# Patch to the spreport.sql script.
	##
	## insert "set newpage 0;" before the line of the ttitles.
	##
	N=`$BIN/echo "$SPRSCR" |
	$USRBIN/wc -l |
	$BIN/awk '{ printf("%d\n",$1+1) }'`
	I=1
	while [ $I -lt $N ]
	do
		TEXT=`$BIN/echo "$SPRSCR" |
		$BIN/awk 'BEGIN { i = '"$I"' }
		{	if ( i == NR )
				printf("%s\n",$0)
			}'`
		$BIN/ed -s spreport.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$TEXT/
		.-1
		a
		set newpage 0;
		.
		w spreport.sql
		q
		EOF
		I=`$USRBIN/expr $I + 1`
	done
	#
	# COR-002 end.
	# COR-003 start.
	# Change for Top 5 Wait Events.
	#
	N=`$BIN/echo "$TOP5SCR" |
	$USRBIN/wc -l |
	$BIN/awk '{ printf("%d\n",$1+1) }'`
	I=1
	while [ $I -lt $N ]
	do
		TEXT=`$BIN/echo "$TOP5SCR" |
		$BIN/awk 'BEGIN { i = '"$I"' }
		{	if ( i == NR )
				printf("%s\n",$0)
			}'`
		$BIN/ed -s spreport.sql 1>$TEMPFILE 2>&1 <<-EOF
		/$TEXT/
		.-2
		a
		set newpage 0
		ttitle lef '$TEXT for ' -
	       		   'DB: ' db_name  '  Instance: ' inst_name '  '-
	       		   'Snaps: ' format 999999 begin_snap ' -' format 999999 end_snap -
	       		skip 2;
		.
		w spreport.sql
		q
		EOF
		I=`$USRBIN/expr $I + 1`
	done
	# COR-003 end.
fi
SetCONNstring perfstat								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	db_name
	from stats\$database_instance;
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
DBNAME=`	$BIN/awk '{	if ( substr($1,1,7) == "DB_NAME" ) {
					getline
					getline
					name=substr($1,1,7)
					}
				}
END { printf("%s\n",name) }' $SPOOLFILE`
# COR-018 start.
#if [ $OS = "CYGWIN" ]; then
#	ORACLE_SID=$ORADO
#fi
# COR-018 end.
# COR-031 start.
if [ $OS = "CYGWIN" ]; then
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select	instance_name from stats\$database_instance;
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
	ORACLE_SID=`
	$BIN/awk '{	if ( $1 == "INSTANCE_NAME" ) {
				getline
				getline
				name=$1
				}
		}
	END { printf("%s\n",name) }' $SPOOLFILE`
fi
# COR-031 end.
# COR-005 start.
SetCONNstring perfstat								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
set LINESIZE 512
set trimspool on
spool $SPOOLFILE
set heading off
select	s.snap_id,
	d.instance_name,
	d.db_name,
	to_char(s.snap_time, 'dd Mon YYYY HH24:mi:ss')
		from stats\$snapshot s, stats\$database_instance d
		where s.dbid = d.dbid
			and s.instance_number = d.instance_number
			and s.startup_time = d.startup_time
			order by snap_id;
show sqlcode;
spool off
EOF
# COR-005 end.
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
$BIN/awk '{	if (( $2 == "'"$ORACLE_SID"'" ) && ( $3 == "'"$DBNAME"'" )) {
			if ( $1 != 0 )
				printf("%d\n",$1)
			if ( $1 == "SQL>" )
				exit
			}
}' $SPOOLFILE | $BIN/sort -n > $TEMPFILE
LINES=`$USRBIN/wc -l $TEMPFILE | awk '{ printf("%d\n",$1) }'`
if [ $LINES -lt 2 ]; then
	LogRecord "ERROR: the number of snaps is $LINES that it needs more than 2."
	ClearfAndExit 1
fi
# COR-026 start. alter session parameters added.
ONCE="YES"
for snap_id in `$BIN/cat $TEMPFILE`
do
	if [ $ONCE = "NO" ]; then
		SID=$EID
		EID=$snap_id
		FDSID=`$BIN/echo $SID | $BIN/awk '{ printf("%04d\n",$1) }'`
		FDEID=`$BIN/echo $EID | $BIN/awk '{ printf("%04d\n",$1) }'`
		SetCONNstring perfstat						#COR-043
		$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF			#COR-043
		alter session set nls_language='AMERICAN';
		alter session set nls_date_format='DD-MON-RR';
		define begin_snap=$SID
		define end_snap=$EID
		define report_name=sp_${FDSID}_${FDEID}.lst
		@spreport
		spool $SPOOLFILE
		show sqlcode
		spool off
		EOF
		CheckSQLcode
	else
		EID=$snap_id
		ONCE="NO"
	fi
	printdot
done
# COR-026 end. alter session parameters added.
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
$BIN/rm $SPOOLFILE
$BIN/rm $TEMPFILE
}

#
# Take STATSPACKreports apart by the sections.
#
spreport () {
##
## The first part of spreport.
## Make up sp_${SNAPBEGINID}_${SNAPENDID}.lst files.
##
GetDBversion									# COR-047
GetORACLEversion								# COR-047
if [ $SPREPORT = "YES" ]; then
	CheckPmon $SID
	CheckUserAccount
	DoSpReportSQL
fi
cd $ORATMP/$LISTDIR
##
## The second part of spreport.
## Make up the individual TOC lists.
##
$BIN/ls sp_*_*.lst 2>/dev/null |
$BIN/grep "sp_" |
$BIN/grep ".lst" 1>/dev/null 2>&1
if [ $? -eq 0 ]; then
	SPFILES=FOUND
else
	SPFILES=NO
fi
if [ $SPFILES = "FOUND" ]; then
	LINES=`$USRBIN/wc -l sp_*_*.lst |
	$BIN/awk '{	lines=$1
	if ( $2 == "total" ) {
		total="YES"
		exit
		}
	else
		n=NR
	}
	END {	if (( total == "YES" ) || ( n == 1 ))
			printf("%d\n",lines)
		else
			printf("NO\n") }'`
	if [ $LINES = "NO" ]; then
		LogRecord "ERROR: any sp_*_*.lst files couldn't be found."
		ClearfAndExit 1
	fi
fi
if [ ! -d $ORATMP/$REPORTDIR ]; then
	$BIN/mkdir $ORATMP/$REPORTDIR
else
	$BIN/rm -rf $ORATMP/$REPORTDIR
	$BIN/mkdir $ORATMP/$REPORTDIR
fi
for i in sp_*_*.lst
do
	if [ ! -f ../date.info ]; then
		SnapDate=`$BIN/awk '{
		if (( $1 == "Begin" ) && ( $2 == "Snap:" )) {
			snapdate=$4
			found="YES"
			exit
			}
		}
		END {	if ( found == "YES" )
				printf("%s\n",snapdate)
			else
				printf("NO\n")
		}' $i`
		if [ $SnapDate != "NO" ]; then
			DateConv $SnapDate
			$BIN/echo $DATESTRING > ../date.info
		fi
	fi
	SNAPDATE=`$BIN/awk '{
		if (( $1 == "Begin" ) && ( $2 == "Snap:" )) {
			split($4,array,"-")
			# COR-034 start.
			year=sprintf("%02d%02d\n","'"$YEARHEAD"'",array[3])
			mc=array[2]
			if ( mc == "Jan" )
				mon=1
			if ( mc == "Feb" )
				mon=2
			if ( mc == "Mar" )
				mon=3
			if ( mc == "Apr" )
				mon=4
			if ( mc == "May" )
				mon=5
			if ( mc == "Jun" )
				mon=6
			if ( mc == "Jul" )
				mon=7
			if ( mc == "Aug" )
				mon=8
			if ( mc == "Sep" )
				mon=9
			if ( mc == "Oct" )
				mon=10
			if ( mc == "Nov" )
				mon=11
			if ( mc == "Dec" )
				mon=12
			month=sprintf("%02d\n",mon)
			date=sprintf("%04d%02d%02d\n",year,month,array[1])
			# COR-034 end.
			exit
			}
		}
		END { printf("%s\n",date) }' $i`
	SNAPSID=`$BIN/awk '{
		if (( $1 == "Begin" ) && ( $2 == "Snap:" )) {
			stime=$3
			exit
			}
		}
		END { printf("%04d\n",stime) }' $i`
	SNAPEID=`$BIN/awk '{
		if (( $1 == "End" ) && ( $2 == "Snap:" )) {
			etime=$3
			exit
			}
		}
		END { printf("%04d\n",etime) }' $i`
	# COR-009 start.
	if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then			#COR-027 COR-043 COR-051
		$BIN/echo "STATSPACK report DB/Inst: $SNAPSID-$SNAPEID" > $ORATMP/$REPORTDIR/toclist
	else
		$BIN/echo "STATSPACK report for db $SNAPSID -$SNAPEID" > $ORATMP/$REPORTDIR/toclist
	fi
	# COR-009 end.
	$BIN/grep "" $i | $BIN/sed 's///g' >> $ORATMP/$REPORTDIR/toclist
	# COR-009 start.
	if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then			#COR-027 COR-043 COR-051
		SEP="DB/Inst:"
	else
		SEP="for"
	fi
	# COR-009 end.
	if [ ${SNAPSID} != "0000" ] && [ ${SNAPEID} != "0000" ]; then
		$BIN/echo "sp_${SNAPDATE}_${SNAPSID}_${SNAPEID}" >> $ORATMP/$REPORTDIR/dirlist
		DIR=$ORATMP/$REPORTDIR/sp_${SNAPDATE}_${SNAPSID}_${SNAPEID}
		if [ ! -d $DIR ]; then
			$BIN/mkdir $DIR
		else
			$BIN/rm $DIR/*
		fi
		$BIN/awk 'BEGIN { suffics=1 } {
		for ( n = 1 ; n < NF ; n++ ) {
			if ( $n == "'"$SEP"'" ) {			#COR-009
				for ( m = 1 ; m < n ; m++ ) {
					name=sprintf("%s_%s",name,$m)
					}
				number=NR
				n--
				numberoffield=n
				n=NF
				p=NF
				split($p,array,"-")
				eno=array[2]
				# COR-009 start.
				if (( "'"$LEVEL"'" == "10" ) || ( "'"$LEVEL"'" == "11" ) || ( "'"$LEVEL"'" == "12" ) || ( "'"$LEVEL"'" == "18" ) || ( "'"$LEVEL"'" == "19" )) {	#COR-027 COR-043 COR-051
					bno=array[1]
					break				#COR-014
					}
				else
					p--
					bno=$p
				# COR-009 end.
			}
		}
		if ( oname == name ) {
			suffics++
			}
		else {
			suffics=1
			oname=name
		}
		numberoffield++
		printf("%04d_%d%s_%d_%04d_%04d\n",number,numberoffield,name,suffics,bno,eno)
		name=""
		}' $ORATMP/$REPORTDIR/toclist > $DIR/toclist
	fi
done
$BIN/rm $ORATMP/$REPORTDIR/toclist
##
## The third part of spreport.
## Take the sp_*_*.lst files apart.
##
for i in `$BIN/cat $ORATMP/$REPORTDIR/dirlist`
do
	TOCLINES=`$USRBIN/wc -l $ORATMP/$REPORTDIR/$i/toclist 2>/dev/null |
	$BIN/awk '{ lines=$1 } END { printf("%d\n",lines) }'`
	for j in `$BIN/cat $ORATMP/$REPORTDIR/$i/toclist`
	do
		LFCOUNT=`$BIN/echo $j |
		$BIN/awk -F_ '{ printf("%d\n",$1) }'`
		FIELDCOUNT=`$BIN/echo $j |
		$BIN/awk -F_ '{ printf("%d\n",$2) }'`
		FILENAME=`$BIN/echo $j |
		$BIN/awk -F_ '
		BEGIN {	i = '$FIELDCOUNT'
			i = i + 3 }
		{	for ( n = 3 ; n < i ; n++ )
				name=sprintf("%s%s",name,$n)	}
		END { printf("%s\n",name) }'`
		FSID=`$BIN/echo $j |
		$BIN/awk -F"_" '{	pos=NF-1
					printf("%04d\n",$pos) }'`
		FEID=`$BIN/echo $j |
		$BIN/awk -F"_" '{ printf("%04d\n",$NF) }'`
		#if [ $FEID = "0000" ]; then
		#	echo $j
		#else
		printdot
		$BIN/awk 'BEGIN {	lfcount=1
					prflag="OFF"
					filecount='$LFCOUNT'
				}
		{
			if ( substr($1,1,1) == "" ) {
				lfcount++
				if ( prflag == "ON" )
					exit
				}
			if ( filecount == lfcount ) {
				prflag="ON"
				}
			if ( prflag == "ON" ) {
				q=split($0,text,"")
				if ( q > 1 )
					printf("%s\n",text[2])
				else
					printf("%s\n",$0)
			}
		}' sp_${FSID}_${FEID}.lst > $ORATMP/$REPORTDIR/$i/$FILENAME
		#fi
	done
	LSTLINES=`$USRBIN/wc -l sp_${FSID}_${FEID}.lst 2>/dev/null |
	$BIN/awk '{ printf("%d\n",$1) }'`
	RESULTLINES=`$USRBIN/wc -l $ORATMP/$REPORTDIR/$i/* 2>/dev/null |
	$BIN/awk '{	if ( $2 == "total" ) {
				lines=$1
				exit
				}
			}
	END {	printf("%d\n",lines) }'`
	N=`$USRBIN/expr $RESULTLINES - $TOCLINES`
	if [ $LSTLINES -eq $N ]; then
		blankline
		LogRecord "INFO:  checking result $i was passed."
	else
		blankline
		echo "DEBUG: $LSTLINES $N"
		LogRecord "ERROR: checking result $i was failed."
	fi
done
$BIN/rm $ORATMP/$REPORTDIR/dirlist
##
## The fourth part of spreport.
## Make up the waitevents.info file for Top5WaitEvents.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
if [ $LEVEL = "10" ] && [ $SUBLEVEL = "2" ]; then
	# COR-024 start.
	cd $ORATMP/$REPORTDIR
	if [ -s $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	for i in `$USRBIN/find * -print | $BIN/grep WaitEvents`
	do
		$BIN/awk '{
		if ( $1 == "Event" ) {
			getline
			getline
			prflag="ON"
			}
		if ( substr($1,1,5) == "-----" )
			exit
		if ( prflag == "ON" )
			printf("%s\n",$0)
		}' $i >>  $TEMPFILE
	done
	if [ -s $TEMPFILE ]; then
		$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
		$BIN/awk ' {
		spc=0
		event=""
		for ( i = 1 ; i <= 33 ; i++ ) {
			if ( substr($0,i,1) != " " ) {
				if ( spc == 1 )
					spc=0
				event=sprintf("%s%s",event,substr($0,i,1))
				}
			else
				{
				spc++
				event=sprintf("%s%s",event,"~")
			}
			if ( spc > 1 )
				break
			}
		printf("%s\n",event)
		}' $WORKFILE > $TEMPFILE
		$BIN/sed -e '
		s/\*/~/g
		s/ /~/g
		s/:/~/g
		s/\//~/g
		s/\~$//g
		s/\~$//g' $TEMPFILE > $ORATMP/waitevents.info
	fi
	# COR-024 end.
else
	# COR-027 start.
	if [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-043 COR-051
		cd $ORATMP/$REPORTDIR
		if [ -s $TEMPFILE ]; then
			$BIN/rm $TEMPFILE
		fi
		for i in `$USRBIN/find * -print | $BIN/grep WaitEvents`
		do
			$BIN/awk '{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if ( substr($1,1,5) == "-----" )
				exit
			if ( prflag == "ON" )
				printf("%s\n",$0)
			}' $i >>  $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
			$BIN/awk ' {
			spc=0
			event=""
			for ( i = 1 ; i <= 33 ; i++ ) {
				if ( substr($0,i,1) != " " ) {
					if ( spc == 1 )
						spc=0
					event=sprintf("%s%s",event,substr($0,i,1))
					}
				else
					{
					spc++
					event=sprintf("%s%s",event,"~")
				}
				if ( spc > 1 )
					break
				}
			printf("%s\n",event)
			}' $WORKFILE > $TEMPFILE
			$BIN/sed -e '
			s/\*/~/g
			s/ /~/g
			s/:/~/g
			s/\//~/g
			s/\~$//g
			s/\~$//g' $TEMPFILE > $ORATMP/waitevents.info
		fi
	else
	# COR-027 end.
		eventnamelength=30
		for i in `$USRBIN/find * -print | $BIN/grep Top5WaitEvents`
		do
			$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
				prflag="OFF"
			if ( prflag == "ON" ) {
				spc=1
				for ( i = 1 ; i <= fieldlength ; i++ ) {
					if (( substr($0,i,1) == " " ) || ( i > length($0))) {
						if ( spc == 1 ) {
							event=sprintf("%s%s",event,"~")
							spc++
							}
						else {
							printf("%s\n",substr(event,1,length(event)-1))
							event=""
							break
							}
						}
					else {
						event=sprintf("%s%s",event,substr($0,i,1))
						spc=1
						}
					}
				}
			}' $i >> $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
			$BIN/sed 's/\//~/g' > $ORATMP/waitevents.info
		fi
	fi
fi
##
## The fifth part of spreport.
## Make up the timedevent.info file for Top5TimedEvents.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
eventnamelength=30
# COR-010 start.
for i in `	$USRBIN/find * -print |
		$BIN/grep TimedEvents |
		$BIN/grep Top`
# COR-010 end.
do
	$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
	{	if ( $1 == "Event" ) {
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if ( prflag == "ON" ) {
			spc=1
			for ( i = 1 ; i <= fieldlength ; i++ ) {
				if ( substr($0,i,1) == " " ) {
					if ( spc == 1 ) {
						event=sprintf("%s%s",event,"~")
						spc++
						}
					else {
						printf("%s\n",substr(event,1,length(event)-1))
						event=""
						break
						}
					}
				else {
					event=sprintf("%s%s",event,substr($0,i,1))
					spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
	$BIN/sed 's/\//~/g' > $ORATMP/timedevent.info
fi
##
## The sixth part of spreport.
## Make up the dictcache.info file for DictionaryCacheStats.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
cachenamelength=25
for i in `$USRBIN/find * -print | $BIN/grep DictionaryCacheStats`
do
	$BIN/awk '{
		if (( $1 == "Cache" ) && ( $2 == "Requests" )) {
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if ( prflag == "ON" ) {
			printf("%s\n",$1)
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE |
	$BIN/sed 's/_/~/g' > $ORATMP/dictcache.info
fi
##
## The seventh part of spreport.
## Make up the libcache.info file for LibraryCacheActivity.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
namespacelength=15
for i in `$USRBIN/find * -print | $BIN/grep LibraryCacheActivity`
do
	$BIN/awk '{
		if (( $1 == "Namespace" ) && ( $2 == "Requests" )) {
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if ( prflag == "ON" ) {
			namespace=$1
			name=substr($0,1,'"$namespacelength"')
			n=split(name,array," ")
			for ( i = 2 ; i <= n ; i++ )
				namespace=sprintf("%s~%s",namespace,array[i])
			printf("%s\n",substr(namespace,1,length(namespace)))
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE > $ORATMP/libcache.info
fi
##
## The eighth part of spreport.
## Make up the latch.info file for LatchActivity.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
if [ $LEVEL = "9" ]; then						#COR-008
	latchnamelength=25
else
	latchnamelength=30
fi
for i in `$USRBIN/find * -print | $BIN/grep LatchActivity`
do
	$BIN/awk 'BEGIN { fieldlength='"$latchnamelength"' + 1  }
	{	if (( $1 == "Latch" ) && (( $2 == "Name" ) || ( $2 == "Requests" ))) {
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if ( prflag == "ON" ) {
			spc=1
			for ( i = 1 ; i <= fieldlength ; i++ ) {
				if (( substr($0,i,1) == " " ) || ( i > length($0))) {
					if ( spc == 1 ) {
						event=sprintf("%s%s",event,"~")
						spc++
						}
					else {
						printf("%s\n",substr(event,1,length(event)-1))
						event=""
						break
						}
					}
				else {
					event=sprintf("%s%s",event,substr($0,i,1))
					spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\//~/g' |
	$BIN/sed 's/ /~/g' > $ORATMP/latch.info
fi
##
## The ninth part of spreport.
## Make up the instance.info file for InstanceActivity.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
statistic=34
for i in `$USRBIN/find * -print | $BIN/grep InstanceActivityStats`
do
	$BIN/awk 'BEGIN { fieldlength='"$statistic"' + 1  }
	{	if (( $1 == "Statistic" ) && (( $2 == "Total" ) || ( $2 == "Begin" ))) { # COR-010
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if ( prflag == "ON" ) {
			spc=1
			for ( i = 1 ; i <= fieldlength ; i++ ) {
				if (( substr($0,i,1) == " " ) || ( i > length($0))) {
					if ( spc == 1 ) {
						event=sprintf("%s%s",event,"~")
						spc++
						}
					else {
						printf("%s\n",substr(event,1,length(event)-1))
						event=""
						break
						}
					}
				else {
					event=sprintf("%s%s",event,substr($0,i,1))
					spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\//~/g' |
	$BIN/sed 's/*/~/g' | $BIN/sed 's/ /~/g' > $ORATMP/instance.info
fi
##
## The tenth part of spreport.
## Make up the sgabreakdown.info file for SGAbreakdowndifference.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
if [ $LEVEL = "9" ]; then						#COR-008
	pool=6
	length=6+30+2
else
	pool=11
	length=11+24+2
fi
for i in `$USRBIN/find * -print | $BIN/grep SGAbreakdowndifference`
do
	$BIN/awk 'BEGIN { fieldlength='"$length"' + 1  }
	{	if (( $1 == "Pool" ) && ( $2 == "Name" )) {
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if ( prflag == "ON" ) {
			spc=1
			for ( i = 1 ; i <= fieldlength ; i++ ) {
				if ( $1 == "log_buffer" ) {
					if ( "'"$LEVEL"'" == "9" )	#COR-008
						printf("~~~~~~~%s\n",$1)
					else
						printf("~~~~~~~~~~~~%s\n",$1)
					break
				}
				if (( $1 == "java" ) && ( $2 == "pool" ) && ( $3 == "free" ) && ( $4 == "memory" )) {
					printf("%s~%s~~~%s~%s\n",$1,$2,$3,$4)
					break
					}
				if (( $1 == "java" ) && ( $2 == "free" ) && ( $3 == "memory" )) {
					printf("%s~~~%s~%s\n",$1,$2,$3)
					break
					}
				if (( $1 == "large" ) && ( $2 == "free" ) && ( $3 == "memory" )) {
					printf("%s~~%s~%s\n",$1,$2,$3)
					break
					}
				if (( $1 == "java" ) && ( $2 == "memory" ) && ( $3 == "in" ) && ( $4 == "use" )) {
					printf("%s~~~%s~%s~%s\n",$1,$2,$3,$4)
					break
					}
				if (( $1 == "large" ) && ( $2 == "pool" ) && ( $3 == "free" ) && ( $4 == "memory" )) {
					printf("%s~%s~~%s~%s\n",$1,$2,$3,$4)
					break
					}
				if (( substr($0,i,1) == " " ) || ( i > length($0))) {
					if ( spc == 1 ) {
						event=sprintf("%s%s",event,"~")
						spc++
						}
					else {
						printf("%s\n",substr(event,1,length(event)-1))
						event=""
						break
						}
					}
				else {
					event=sprintf("%s%s",event,substr($0,i,1))
					spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\//~/g' |
	$BIN/awk '{	if ( length($0) != 0 ) printf("%s\n",$0) }' |
	$BIN/sed 's/*/~/g' | $BIN/sed 's/ /~/g' > $ORATMP/sgabreakdown.info
fi
##
## The eleventh part of spreport.
## Make up the waitevents.info file for WaitEvents section.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
if [ $LEVEL = "10" ] && [ $SUBLEVEL = "2" ]; then
	# COR-024 start.
	cd $ORATMP/$REPORTDIR
	if [ -s $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	for i in `$USRBIN/find * -print | $BIN/grep WaitEvents`
	do
		$BIN/awk 'BEGIN { fieldlength=33 }
		{
		if ( $1 == "Event" ) {
			getline
			getline
			prflag="ON"
			}
		if ( substr($1,1,5) == "-----" )
			exit
		if ( prflag == "ON" )
			printf("%s\n",$0)
		}' $i >>  $TEMPFILE
	done
	if [ -s $TEMPFILE ]; then
		$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
		$BIN/awk ' {
		spc=0
		event=""
		for ( i = 1 ; i <= 33 ; i++ ) {
			if ( substr($0,i,1) != " " ) {
				if ( spc == 1 )
					spc=0
				event=sprintf("%s%s",event,substr($0,i,1))
				}
			else
				{
				spc++
				event=sprintf("%s%s",event,"~")
			}
			if ( spc > 1 )
				break
			}
		printf("%s\n",event)
		}' $WORKFILE > $TEMPFILE
		$BIN/sed -e '
		s/\*/~/g
		s/ /~/g
		s/:/~/g
		s/\//~/g
		s/\~$//g
		s/\~$//g' $TEMPFILE > $ORATMP/waitevents.info
	fi
	# COR-024 end.
else
	# COR-027 start.
	if [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-043 COR-051
		cd $ORATMP/$REPORTDIR
		if [ -s $TEMPFILE ]; then
			$BIN/rm $TEMPFILE
		fi
		for i in `$USRBIN/find * -print | $BIN/grep WaitEvents`
		do
			$BIN/awk 'BEGIN { fieldlength=33 }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if ( substr($1,1,5) == "-----" )
				exit
			if ( prflag == "ON" )
				printf("%s\n",$0)
			}' $i >>  $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
			$BIN/awk ' {
			spc=0
			event=""
			for ( i = 1 ; i <= 33 ; i++ ) {
				if ( substr($0,i,1) != " " ) {
					if ( spc == 1 )
						spc=0
					event=sprintf("%s%s",event,substr($0,i,1))
					}
				else
					{
					spc++
					event=sprintf("%s%s",event,"~")
				}
				if ( spc > 1 )
					break
				}
			printf("%s\n",event)
			}' $WORKFILE > $TEMPFILE
			$BIN/sed -e '
			s/\*/~/g
			s/ /~/g
			s/:/~/g
			s/\//~/g
			s/\~$//g
			s/\~$//g' $TEMPFILE > $ORATMP/waitevents.info
		fi
	else
	# COR-027 end.
		eventnamelength=30
		for i in `$USRBIN/find * -print | $BIN/grep WaitEvents`
		do
			$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
				prflag="OFF"
			if ( prflag == "ON" ) {
				spc=1
				for ( i = 1 ; i <= fieldlength ; i++ ) {
					if (( substr($0,i,1) == " " ) || ( i > length($0))) {
						if ( spc == 1 ) {
							event=sprintf("%s%s",event,"~")
							spc++
							}
						else {
							printf("%s\n",substr(event,1,length(event)-1))
							event=""
							break
							}
						}
					else {
						event=sprintf("%s%s",event,substr($0,i,1))
						spc=1
						}
					}
				}
			}' $i >> $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
			$BIN/sed 's/\//~/g' > $ORATMP/waitevents.info
		fi
	fi
fi
##
## The twelfth part of spreport.
## Make up the bwaitevents.info file for BackgroundWaitEvents section,
## and ForegroundWaitEvents section(Oracle11g).				#COR-027
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
if [ $LEVEL = "10" ] && [ $SUBLEVEL = "2" ]; then
	# COR-024 start.
	cd $ORATMP/$REPORTDIR
	if [ -s $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	for i in `$USRBIN/find * -print | $BIN/grep BackgroundWaitEvents`
	do
		$BIN/awk 'BEGIN { fieldlength=33  }
		{
		if ( $1 == "Event" ) {
			getline
			getline
			prflag="ON"
			}
		if ( substr($1,1,5) == "-----" )
			exit
		if ( prflag == "ON" )
			printf("%s\n",$0)
		}' $i >>  $TEMPFILE
	done
	if [ -s $TEMPFILE ]; then
		$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
		$BIN/awk ' {
		spc=0
		event=""
		for ( i = 1 ; i <= 33 ; i++ ) {
			if ( substr($0,i,1) != " " ) {
				if ( spc == 1 )
					spc=0
				event=sprintf("%s%s",event,substr($0,i,1))
				}
			else
				{
				spc++
				event=sprintf("%s%s",event,"~")
			}
			if ( spc > 1 )
				break
			}
		printf("%s\n",event)
		}' $WORKFILE > $TEMPFILE
		$BIN/sed -e '
		s/\*/~/g
		s/ /~/g
		s/:/~/g
		s/\//~/g
		s/\~$//g
		s/\~$//g' $TEMPFILE > $ORATMP/bwaitevents.info
	fi
	# COR-024 end.
else
	# COR-027 start.
	if [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-043 COR-051
		cd $ORATMP/$REPORTDIR
		if [ -s $TEMPFILE ]; then
			$BIN/rm $TEMPFILE
		fi
		for i in `$USRBIN/find * -print | $BIN/grep BackgroundWaitEvents`
		do
			$BIN/awk 'BEGIN { fieldlength=33  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if ( substr($1,1,5) == "-----" )
				exit
			if ( prflag == "ON" )
				printf("%s\n",$0)
			}' $i >>  $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
			$BIN/awk ' {
			spc=0
			event=""
			for ( i = 1 ; i <= 33 ; i++ ) {
				if ( substr($0,i,1) != " " ) {
					if ( spc == 1 )
						spc=0
					event=sprintf("%s%s",event,substr($0,i,1))
					}
				else
					{
					spc++
					event=sprintf("%s%s",event,"~")
				}
				if ( spc > 1 )
					break
				}
			printf("%s\n",event)
			}' $WORKFILE > $TEMPFILE
			$BIN/sed -e '
			s/\*/~/g
			s/ /~/g
			s/:/~/g
			s/\//~/g
			s/\~$//g
			s/\~$//g' $TEMPFILE > $ORATMP/bwaitevents.info
		fi
		$BIN/rm $TEMPFILE
		for i in `$USRBIN/find * -print | $BIN/grep ForegroundWaitEvents`
		do
			$BIN/awk 'BEGIN { fieldlength=33  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if ( substr($1,1,5) == "-----" )
				exit
			if ( prflag == "ON" )
				printf("%s\n",$0)
			}' $i >>  $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/cut -b1-33 $TEMPFILE | $BIN/sort -u > $WORKFILE
			$BIN/awk ' {
			spc=0
			event=""
			for ( i = 1 ; i <= 33 ; i++ ) {
				if ( substr($0,i,1) != " " ) {
					if ( spc == 1 )
						spc=0
					event=sprintf("%s%s",event,substr($0,i,1))
					}
				else
					{
					spc++
					event=sprintf("%s%s",event,"~")
				}
				if ( spc > 1 )
					break
				}
			printf("%s\n",event)
			}' $WORKFILE > $TEMPFILE
			$BIN/sed -e '
			s/\*/~/g
			s/ /~/g
			s/:/~/g
			s/\//~/g
			s/\~$//g
			s/\~$//g' $TEMPFILE > $ORATMP/fwaitevents.info
		fi
	else
	# COR-027 end.
		eventnamelength=29
		for i in `$USRBIN/find * -print | $BIN/grep BackgroundWaitEvents`
		do
			$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
				prflag="OFF"
			if ( prflag == "ON" ) {
				spc=1
				for ( i = 1 ; i <= fieldlength ; i++ ) {
					if (( substr($0,i,1) == " " )  || ( i > length($0))) {
						if ( spc == 1 ) {
							event=sprintf("%s%s",event,"~")
							spc++
							}
						else {
							printf("%s\n",substr(event,1,length(event)-1))
							event=""
							break
							}
						}
					else {
						event=sprintf("%s%s",event,substr($0,i,1))
						spc=1
						}
					}
				}
			}' $i >> $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
			$BIN/sed 's/\//~/g' > $ORATMP/bwaitevents.info
		fi
	fi
fi
#
# COR-010 start.
##
## The thirteenth  part of spreport.
## Make up the osstats.info file for OSStatistics.
##
if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-027 COR-043 COR-051
	cd $ORATMP/$REPORTDIR
	if [ -s $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	statistic=25
	for i in `$USRBIN/find * -print | $BIN/grep OSStatistics`
	do
			$BIN/awk '{
			if (( $1 == "Statistic" ) && ( $2 == "Total" )) {
				getline
				getline
				prflag="ON"
				}
			if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
				prflag="OFF"
			if ( prflag == "ON" ) {
				printf("%s\n",$1)
			}
		}' $i >> $TEMPFILE
	done
	if [ -s $TEMPFILE ]; then
		$BIN/sort -u $TEMPFILE |
		$BIN/sed 's/_/~/g' > $ORATMP/osstats.info
	fi
fi
##
## The fourteenth part of spreport.
## Make up the timemodel.info file for TimeModelSystemStats section.
##
if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-027 COR-043 COR-051
	cd $ORATMP/$REPORTDIR
	if [ -s $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	statistic=36
	for i in `$USRBIN/find * -print | $BIN/grep TimeModelSystemStats`
	do
		$BIN/awk 'BEGIN { fieldlength='"$statistic"' + 1  }
		{	if ( $1 == "Statistic" ) {
				getline
				getline
				prflag="ON"
			}
			if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
				prflag="OFF"
			if ( prflag == "ON" ) {
				spc=1
				for ( i = 1 ; i <= fieldlength ; i++ ) {
				if (( substr($0,i,1) == " " ) || ( i > length($0))) {
					if ( spc == 1 ) {
						event=sprintf("%s%s",event,"~")
						spc++
						}
					else {
						printf("%s\n",substr(event,1,length(event)-1))
						event=""
						break
						}
					}
				else {
					event=sprintf("%s%s",event,substr($0,i,1))
					spc=1
					}
				}
			}
		}' $i >> $TEMPFILE
	done
	if [ -s $TEMPFILE ]; then
		$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
		$BIN/sed 's/\//~/g' > $ORATMP/timemodel.info
	fi
fi
##
## The fifteenth part of spreport.
## Make up the waitevhistogram.info file for WaitEventHistogram section.
##
if [ $LEVEL = "10" ] && [ $SUBLEVEL != "2" ]; then			#COR-024
	cd $ORATMP/$REPORTDIR
	if [ -s $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	eventnamelength=50
	for i in `$USRBIN/find * -print | $BIN/grep WaitEventHistogram`
	do
		$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
		{
		if ( $1 == "Event" ) {
			getline
			getline
			getline
			getline
			prflag="ON"
			}
		if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
			prflag="OFF"
		if (( prflag == "ON" ) && ( $1 !~ /[0123456789]/ )) {
			spc=1
			for ( i = 1 ; i <= fieldlength ; i++ ) {
				if (( substr($0,i,1) == " " ) || ( i > length($0))) {
					if ( spc == 1 ) {
						event=sprintf("%s%s",event,"~")
						spc++
						}
					else {
						printf("%s\n",substr(event,1,length(event)-1))
						event=""
						break
						}
					}
				else {
					event=sprintf("%s%s",event,substr($0,i,1))
					spc=1
					}
				}
			}
		}' $i >> $TEMPFILE
	done
	if [ -s $TEMPFILE ]; then
		$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
		$BIN/sed 's/\//~/g' > $ORATMP/waitevhistogram.info
	fi
	# COR-024 end.
else
	# COR-027 start.
	if [ $LEVEL = "11" ] || [ $LEVEL = "12" ]  || [ $LEVEL = "18" ]  || [ $LEVEL = "19" ]; then	#COR-043 COR-051
		cd $ORATMP/$REPORTDIR
		if [ -s $TEMPFILE ]; then
			$BIN/rm $TEMPFILE
		fi
		eventnamelength=50
		for i in `$USRBIN/find * -print | $BIN/grep WaitEventHistogram`
		do
			$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				getline
				getline
				prflag="ON"
				}
			if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
				prflag="OFF"
			if (( prflag == "ON" ) && ( $1 !~ /[0123456789]/ )) {
				spc=1
				for ( i = 1 ; i <= fieldlength ; i++ ) {
					if (( substr($0,i,1) == " " ) || ( i > length($0))) {
						if ( spc == 1 ) {
							event=sprintf("%s%s",event,"~")
							spc++
							}
						else {
							printf("%s\n",substr(event,1,length(event)-1))
							event=""
							break
							}
						}
					else {
						event=sprintf("%s%s",event,substr($0,i,1))
						spc=1
						}
					}
				}
			}' $i >> $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
			$BIN/sed 's/\//~/g' > $ORATMP/waitevhistogram.info
		fi
	else
	# COR-027 end.
		cd $ORATMP/$REPORTDIR
		if [ -s $TEMPFILE ]; then
			$BIN/rm $TEMPFILE
		fi
		for i in `$USRBIN/find * -print | $BIN/grep WaitEventHistogram`
		do
			$BIN/awk 'BEGIN { fieldlength='"$eventnamelength"' + 1  }
			{
			if ( $1 == "Event" ) {
				getline
				getline
				prflag="ON"
				}
			if ( substr($1,1,5) == "-----" )
				exit
			if ( prflag == "ON" )
				printf("%s\n",$0)
			}' $i >>  $TEMPFILE
		done
		if [ -s $TEMPFILE ]; then
			$BIN/cut -b1-26 $TEMPFILE | $BIN/sort -u > $WORKFILE
			$BIN/awk ' {
			spc=0
			event=""
			for ( i = 1 ; i <= 26 ; i++ ) {
				if ( substr($0,i,1) != " " ) {
					if ( spc == 1 )
						spc=0
					event=sprintf("%s%s",event,substr($0,i,1))
					}
				else
					{
					spc++
					event=sprintf("%s%s",event,"~")
				}
				if ( spc > 1 )
					break
				}
			printf("%s\n",event)
			}' $WORKFILE > $TEMPFILE
			$BIN/sed -e '
			s/\*/~/g
			s/ /~/g
			s/:/~/g
			s/\//~/g
			s/\~$//g
			s/\~$//g' $TEMPFILE > $ORATMP/waitevhistogram.info
		fi
	fi
fi
# COR-024 end.
# COR-010 end.
# COR-015 start.
##
## The sixteenth part of spreport.
## Make up the bufferwaitclass.info file for BufferwaitStatistics section.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
# COR-044 start.
if [ $LEVEL = "10" ] || [ $LEVEL = "11" ] || [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	# COR-051
	classlength=22
else
	classlength=18
fi
# COR-044 end.
for i in `$USRBIN/find * -print | $BIN/grep BufferwaitStatistics`
do
	$BIN/awk 'BEGIN { fieldlength='"$classlength"' + 1  }
	{
	if ( $1 == "Class" ) {
		getline
		getline
		prflag="ON"
		}
	if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
		prflag="OFF"
	if (( prflag == "ON" ) && ( $1 !~ /[0123456789]/ )) {
		spc=1
		for ( i = 1 ; i <= fieldlength ; i++ ) {
			if (( substr($0,i,1) == " " ) || ( i > length($0))) {
				if ( spc == 1 ) {
					class=sprintf("%s%s",class,"~")
					spc++
					}
				else {
					printf("%s\n",substr(class,1,length(class)-1))
					class=""
					break
					}
				}
			else {
				class=sprintf("%s%s",class,substr($0,i,1))
				spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
	$BIN/sed 's/\//~/g' > $ORATMP/bufferwaitclass.info
fi
# COR-015 end.
# COR-035 start.
##
## The seventeenth part of spreport.
## Make up the globalcrserved.info file for GlobalCRServedStats section.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
statisticlength=30
for i in `$USRBIN/find * -print | $BIN/grep GlobalCRServedStats`
do
	$BIN/awk 'BEGIN { fieldlength='"$statisticlength"' + 1  }
	{
	if ( $1 == "Statistic" ) {
		getline
		getline
		prflag="ON"
		}
	if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
		prflag="OFF"
	if (( prflag == "ON" ) && ( $1 !~ /[0123456789]/ )) {
		spc=1
		for ( i = 1 ; i <= fieldlength ; i++ ) {
			if (( substr($0,i,1) == " " ) || ( i > length($0))) {
				if ( spc == 1 ) {
					statistic=sprintf("%s%s",statistic,"~")
					spc++
					}
				else {
					printf("%s\n",substr(statistic,1,length(statistic)-1))
					statistic=""
					break
					}
				}
			else {
				statistic=sprintf("%s%s",statistic,substr($0,i,1))
				spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
	$BIN/sed 's/\//~/g' > $ORATMP/globalcrserved.info
fi
# COR-035 end.
#
# COR-036 start.
##
## The eighteenth part of spreport.
## Make up the globalenqueue.info file for GlobalEnqueueStats section.
##
cd $ORATMP/$REPORTDIR
if [ -s $TEMPFILE ]; then
	$BIN/rm $TEMPFILE
fi
statisticlength=34
for i in `$USRBIN/find * -print | $BIN/grep GlobalEnqueueStats`
do
	$BIN/awk 'BEGIN { fieldlength='"$statisticlength"' + 1  }
	{
	if ( $1 == "Statistic" ) {
		getline
		getline
		prflag="ON"
		}
	if (( prflag == "ON" ) && ( substr($1,1,1) == "-" ))
		prflag="OFF"
	if (( prflag == "ON" ) && ( $1 !~ /[0123456789]/ )) {
		spc=1
		for ( i = 1 ; i <= fieldlength ; i++ ) {
			if (( substr($0,i,1) == " " ) || ( i > length($0))) {
				if ( spc == 1 ) {
					statistic=sprintf("%s%s",statistic,"~")
					spc++
					}
				else {
					printf("%s\n",substr(statistic,1,length(statistic)-1))
					statistic=""
					break
					}
				}
			else {
				statistic=sprintf("%s%s",statistic,substr($0,i,1))
				spc=1
				}
			}
		}
	}' $i >> $TEMPFILE
done
if [ -s $TEMPFILE ]; then
	$BIN/sort -u $TEMPFILE | $BIN/sed 's/\*/~/g' |
	$BIN/sed 's/\//~/g' > $ORATMP/globalenqueue.info
fi
# COR-036 end.
}
#
# end spreport.
#

#
# Get tablespace information.
#
GetTableSpace () {
if [ ! -s $ORATMP/tablespaces.info ]; then
	SetCONNstring system							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select	tablespace_name from dba_tablespaces;
	show sqlcode;
	spool off
	EOF
	CheckSQLcode
	$BIN/mv $SPOOLFILE $ORATMP/tablespaces.info
	$BIN/rm $TEMPFILE
	for table in $TOOLS TEMP					#COR-021
	do
		TABLE=`$BIN/awk '{	if ( $1 == "'"$table"'" ) {
					found="YES"
					exit
					}
				}
			END {	if ( found == "YES" )
					printf("%s\n",found)
				else
					printf("NO\n")
			}' $ORATMP/tablespaces.info`
		if [ $TABLE = "NO" ]; then
			LogRecord "ERROR: $table tablespace not found."
			ClearfAndExit 1
		fi
	done
fi
}

#
# Get ORACLE version from the DataBase.
#
GetDBversion () {
if [ ! -s $ORATMP/version.info ]; then
	$ORABIN/sqlplus system/$SYSTEMPASSWORD 1>$TEMPFILE 2>&1 <<-EOF
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select version from v\$instance;
	show sqlcode;
	spool off
	EOF
	CheckSQLcode
	$BIN/mv $SPOOLFILE $ORATMP/version.info
	$BIN/rm $TEMPFILE
fi
}

#
# Get ORACLE version from the version.info file.
#
GetORACLEversion () {
if [ -s $ORATMP/version.info ]; then
	VERSION=`
	$BIN/awk '{	if ( $1 == "VERSION" ) {
				getline
				getline
				version=$1
				found="YES"
				exit
				}
		}
	END {	if ( found == "YES" )
			printf("%s\n",version)
		else
			printf("NO\n")
	}' $ORATMP/version.info`
	if [ $VERSION = "NO" ]; then
		LogRecord "ERROR: ORACLE version not found."
		ClearfAndExit 1
	fi
else
	VERSION=$LEVEL102010							#COR-043
	LogRecord "NOTE:  version.info file not found. $VERSION was assumed."
fi
RELEASE=`$BIN/echo $VERSION | $BIN/awk -F"." '{ printf("%d\n",$1) }'`
SUBLEVEL=`$BIN/echo $VERSION | $BIN/awk -F"." '{ printf("%d\n",$2) }'`	#COR-024
# COR-008 start.
case $RELEASE in
19|18|12|11|10|9|8)	LEVEL=$RELEASE	;;				#COR-027 COR-043 COR-051
*)	LogRecord "ERROR: version number $RELEASE unknown."
	ClearfAndExit 1	;;						#COR-017
# COR-008 end.
esac
# COR-043 start.
test ${PDBNAME}
if [ $? -eq 0 ]; then
	# COR-051 start.
	if [ $LEVEL != "12" ] && [ $LEVEL != "18" ] && [ $LEVEL != "19" ]; then
		LogRecord "ERROR: ORACLE is not 12c,18c,19c PDBNAME variable must be unset."
	# COR-051 end.
		ClearfAndExit 1
	fi
else
	if [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-051
		LogRecord "ERROR: ORACLE 12c needs PDBNAME variable."
		ClearfAndExit 1
	fi
fi
# COR-051 start.
if [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then
	$ORABIN/sqlplus system/$SYSTEMPASSWORD@localhost/$PDBNAME 1>$TEMPFILE 2>&1 <<-EOF
# COR-051 end.
	alter session set nls_language='AMERICAN';
	set sqlcase lower
	spool $SPOOLFILE
	select name,open_mode from v\$pdbs;
	show sqlcode;
	spool off
	EOF
	CheckSQLcode
	$BIN/mv $SPOOLFILE $ORATMP/pdb.info
	$BIN/rm $TEMPFILE
	PDBMODE=`$BIN/$nawk 'BEGIN {
	pdbname=toupper("'"$PDBNAME"'")
	mode="NO"
	}
	{
	if ( $1 == pdbname ) {
		# COR-050 start.
		if (( "'"$VERSION"'" == "12.2.0.1.0" ) || ( "'"$VERSION"'" == "18.0.0.0.0" ) || ( "'"$VERSION"'" == "19.0.0.0.0" )) {	# COR-051
			getline
			mode=sprintf("%s%s",$1,$2)
			}
		else {
			mode=sprintf("%s%s",$2,$3)
			}
		# COR-050 end.
		}
	} END {
	printf("%s\n",mode)
	}' $ORATMP/pdb.info`
	if [ $PDBMODE != "READWRITE" ]; then
		LogRecord "ERROR: the pluggable database $PDBNAME open_mode is "$PDBMODE"."
		ClearfAndExit 1
	fi
fi
# COR-043 end.
}

#
# List contents of splistdir.
#
splists () {
arg1=$1
arg2=$2
if [ "${arg1}_x" != "_x" ] && [ "${arg2}_x" != "_x" ]; then
	ORATMP=${LOGDIR}/orastats_${arg1}_${arg2}
fi
if [ ! -d $ORATMP/$LISTDIR ]; then
	LogRecord "ERROR: splistdir not found."
	ClearfAndExit 1
fi
cd $ORATMP/$LISTDIR
N=1
$BIN/echo "        No.      spreportFileName    SnapInDate     BegnTime    EndTime." > $TEMPFILE
$BIN/echo "        ----     ----------------    ----------     --------    --------" >> $TEMPFILE
for i in `$BIN/ls | $BIN/grep "sp_...._.....lst"`
do
	SnapDate=`$BIN/awk '{
	if (( $1 == "Begin" ) && ( $2 == "Snap:" )) {
		found="YES"
		snapdate=$4
		exit
		}
	}
	END {	if ( found == "YES")
			printf("%s\n",snapdate)
		else
			printf("NO\n")
	}' $i`
	DateConv $SnapDate
	TimeInfo=`$BIN/awk '{
	if (( $1 == "Begin" ) && ( $2 == "Snap:" )) {
		found="YES"
		begintime=$5
		getline
		endtime=$5
		exit
		}
	}
	END {	if ( found == "YES" )
			printf("    %s    %s\n",begintime,endtime)
		else
			printf("\n")
	}' $i`
	NUMBER=`$BIN/echo $N | $BIN/awk '{ printf("%4d\n",'"$N"') }'`
	$BIN/echo "        $NUMBER     $i    $DATESTRING $TimeInfo" >> $TEMPFILE
	N=`$USRBIN/expr $N + 1`
done
$BIN/echo "" >> $TEMPFILE
$BIN/echo "        END REPORT" >> $TEMPFILE
$BIN/more $TEMPFILE
$BIN/rm $TEMPFILE
}

#
# Create PERFSTAT user account, tables, and packages.
#
spsetup() {
SPFILELIST="
spcreate.sql
spcusr.sql
spdusr.sql
dbmspool.sql
prvtpool.plb
dbmsjob.sql
spctab.sql
spcpkg.sql
"
GetDBversion									#COR-046
GetORACLEversion								#COR-045
cd $ORATMP
CheckPmon $SID
CheckUserAccount RETURN
if [ $USERACTIVE = "YES" ]; then
	LogRecord "ERROR: perfstat user account has already been created."
	ClearfAndExit 1
fi
for f in `$BIN/echo $SPFILELIST`
do
	if [ -s $ADMINDIR/${f} ]; then
		$BIN/cp $ADMINDIR/${f} ${f}
		if [ $? -ne 0 ]; then		# COR-038
			LogRecord "ERROR: ${f} copy failed."
			ClearfAndExit 1
		fi
	else
		LogRecord "ERROR: ${f} not found."
	fi
done
# COR-043 start.
if [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	# COR-051
	# COR-053 start.
	$BIN/ed -s spcusr.sql 1>$TEMPFILE 2>&1 <<-EOF
	/grant execute on DBMS_JOB/
	.
	a
	grant execute on dbms_scheduler to perfstat;
	grant manage scheduler to perfstat;
	grant create job to perfstat;
	.
	w spcusr.sql
	q
	EOF
	# COR-053 end.

	# COR-054 start.
		if [ $LEVEL = "18" ]; then
		$BIN/ed -s spcpkg.sql 1>$TEMPFILE 2>&1 <<-EOF
		/eocur := GET_SYSSTAT/
		.
		a
		if eocur = 0 then
		  eocur := 1;
		end if;
		.
		w spcpkg.sql
		q
		EOF
		$BIN/ed -s spcpkg.sql 1>$TEMPFILE 2>&1 <<-EOF
		/bocur := GET_SYSSTAT/
		.
		a
		if bocur = 0 then
		  bocur := 1;
		end if;
		.
		w spcpkg.sql
		q
		EOF
		$BIN/ed -s spcpkg.sql 1>$TEMPFILE 2>&1 <<-EOF
		/elog := GET_SYSSTAT/
		.
		a
		if elog = 0 then
		  elog := 1;
		end if;
		.
		w spcpkg.sql
		q
		EOF
		$BIN/ed -s spcpkg.sql 1>$TEMPFILE 2>&1 <<-EOF
		/blog := GET_SYSSTAT/
		.
		a
		if blog = 0 then
		  blog := 1;
		end if;
		.
		w spcpkg.sql
		q
		EOF
	fi
	# COR-054 end.
	SetCONNstring system
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
	connect / as sysdba
	alter session set container = $PDBNAME;
	alter session set nls_language='AMERICAN';
	define perfstat_password='perfstat'
	define default_tablespace='$tools'
	define temporary_tablespace='temp'
	@spcreate
	grant connect,resource to perfstat;
	spool $SPOOLFILE
	show sqlcode;
	spool off
	EOF
else
	# COR-045 start.
	if [ $LEVEL = "8" ]; then
		SetCONNstring system
		$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
		connect / as sysdba
		@spdusr
		EOF
	fi
	# COR-045 end.
	SetCONNstring system
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
	connect / as sysdba
	alter session set nls_language='AMERICAN';
	define perfstat_password='perfstat'
	define default_tablespace='$tools'
	define temporary_tablespace='temp'
	@spcreate
	spool $SPOOLFILE
	show sqlcode;
	spool off
	EOF
fi
# COR-043 end.
$BIN/cat spcusr.lis >> $ORATMP/$FUNCTION.log 1>/dev/null 2>&1
$BIN/cat spctab.lis >> $ORATMP/$FUNCTION.log 1>/dev/null 2>&1
$BIN/cat spcpkg.lis >> $ORATMP/$FUNCTION.log 1>/dev/null 2>&1
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log 1>/dev/null 2>&1
CheckSQLcode
#
# Get tablespace information.
GetTableSpace
#
# Get ORACLE version.
GetDBversion
LogRecord "INFO:  perfstat user account was successfully created."
}

#
# Clean up PERFSTAT tables, packages, and user account.
#
spclean() {
SPFILELIST="
spdtab.sql
spdusr.sql
"
cd $ORATMP
CheckPmon $SID
GetDBversion									#COR-047
GetORACLEversion								#COR-047
GetJobNumber RETURN
if [ $USERACTIVE = "NO" ] && [ $JOBNO = "NO" ]; then
	LogRecord "ERROR: perfstat user account has already been cleaned up."
	ClearfAndExit 1
fi
if [ $JOBNO != "NO" ]; then
	LogRecord "WARN:  job number $JOBNO is still running."
	ClearfAndExit 1
fi
for f in `$BIN/echo $SPFILELIST`
do
	if [ -s $ADMINDIR/${f} ]; then
		$BIN/cp $ADMINDIR/${f} ${f}
		if [ $? -ne 0 ]; then		# COR-038
			LogRecord "ERROR: ${f} copy failed."
			ClearfAndExit 1
		fi
	else
		LogRecord "ERROR: ${f} not found."
		ClearfAndExit 1
	fi
done

# COR-052 start.
$BIN/ed -s spdusr.sql 1>$TEMPFILE 2>&1 <<-EOF
/drop user perfstat cascade;/
.-1
a
alter session set "_oracle_script"=true;
.
w spdusr.sql
q
EOF
# COR-052 end..
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
@spdtab
@spdusr
spool $SPOOLFILE
show sqlcode
spool off
EOF
$BIN/cat spdtab.lis >> $ORATMP/$FUNCTION.log
$BIN/cat spdusr.lis >> $ORATMP/$FUNCTION.log
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
LogRecord "INFO:  perfstat user was successfully cleaned up."
}

#
# The SQL is collecting STATSPACK statistics automatically.
# This part starts that the SQL*PLUS script spauto.sql.
#
spstart () {
cd $ORATMP
INTERVAL=$1
if [ "${INTERVAL}_x" = "_x" ]; then
	LogRecord "ERROR: start function needs time interval."
	ClearfAndExit 1
fi
CheckPmon $SID
GetDBversion									# COR-047
GetORACLEversion								# COR-047
GetJobNumber RETURN
if [ $USERACTIVE = "NO" ]; then
	LogRecord "ERROR: perfstat user account hasn't been created."
	ClearfAndExit 1
fi
if [ $JOBNO != "NO" ]; then
	LogRecord "ERROR: job number $JOBNO has been running."
	ClearfAndExit 1
fi
ONEDAYMIN=1440
ONEDAYHOUR=24
NTYPE=MI
if [ $INTERVAL -eq 0 ]; then
	LogRecord "ERROR: time interval should be more than one."
	ClearfAndExit 1
fi
TRUNCM=`$USRBIN/expr $ONEDAYMIN \/ $INTERVAL`
if [ $TRUNCM = $ONEDAYHOUR ]; then
	NTYPE="HH"
fi
$BIN/cp $ADMINDIR/spauto.sql spauto.sql
$BIN/ed -s spauto.sql 1>/dev/null 2>&1 <<-EOF
/dbms_job.submit/
s/24/$TRUNCM/g
s/HH/$NTYPE/g
1,$ w spauto.sql
q
EOF
SetCONNstring perfstat								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
execute statspack.snap;
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
SetCONNstring perfstat								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
@spauto
spool $SPOOLFILE
show sqlcode
spool off
EOF
$BIN/cat spauto.lis >> $ORATMP/$FUNCTION.log
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
LogRecord "INFO:  statspack spauto job was successfully scheduled."
}

#
# Stop the JOB that it has been started by the spauto.sql SQL*PLUS script.
#
spstop () {
cd $ORATMP
CheckPmon $SID
GetJobNumber RETURN
SLEEPTIME=10
if [ $USERACTIVE = "NO" ]; then
	LogRecord "ERROR: perfstat user account hasn't been created."
	ClearfAndExit 1
fi
if [ $JOBNO = "NO" ]; then
	LogRecord "ERROR: statspack job hasn't been started."
	ClearfAndExit 1
fi
GetNextSec
S2I $NEXT_SEC
DBTIME=$TIMEVALUE
CURTIME=`$BIN/date '+%H:%M:%S'`
S2I $CURTIME
TIMEDIFF=`$USRBIN/expr $DBTIME - $TIMEVALUE`
# $BIN/echo "Nexttime = $NEXT_SEC dbtime = $DBTIME"
# $BIN/echo "curtime  = $CURTIME  cursec = $TIMEVALUE timediff = $TIMEDIFF"
if [ $TIMEDIFF -gt $SLEEPTIME ]; then
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	execute dbms_job.remove($JOBNO);
	execute statspack.snap;
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
else
	TIMEDIFF=`$USRBIN/expr $TIMEDIFF + $SLEEPTIME`
	LogRecord "INFO:  waiting settledown the last snap for $TIMEDIFF sec."
	$BIN/sleep $TIMEDIFF
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	execute dbms_job.remove($JOBNO);
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
fi
LogRecord "INFO:  statspack spauto job was stopped."
}

#
# Export the whole STATSPACK user.
#
spexport () {
GetDBversion									#COR-047
GetORACLEversion								#COR-047
cd $ORATMP
if [ -s $ADMINDIR/spuexp.par ]; then
	$BIN/cp $ADMINDIR/spuexp.par spuexp.par
	if [ $? -ne 0 ]; then		# COR-038
		LogRecord "ERROR: spuexp.par copy failed."
		ClearfAndExit 1
	fi
else
	LogRecord "ERROR: spuexp.par not found."
	ClearfAndExit 1
fi
CheckPmon $SID
GetJobNumber RETURN
if [ $USERACTIVE = "NO" ]; then
	LogRecord "ERROR: perfstat user account hasn't been created."
	ClearfAndExit 1
fi
if [ $JOBNO != "NO" ]; then
	LogRecord "ERROR: statspack job, number $JOBNO is running."
	ClearfAndExit 1
fi
SetCONNstring perfstat								#COR-043
$ORABIN/exp userid=$CONN parfile=spuexp.par 1>$ORATMP/$FUNCTION.log 2>&1	#COR-043
if [ $? -ne 0 ]; then
	LogRecord "ERROR: the return status of exp command wasn't zero."
	LogRecord "WARN:  check $ORATMP/export.log."
	$BIN/mv spuexp.dmp $ORATMP/spuexp.dmp.`$BIN/date '+%y%m%d%H%M%S'` #COR-022
	ClearfAndExit 1
fi
$BIN/mv spuexp.dmp $ORATMP/spuexp.dmp.`$BIN/date '+%y%m%d%H%M%S'`
LogRecord "INFO:  export perfstat was successfully completed."
}

#
# Import the whole STATSPACK user.
#
spimport() {
IMPEXECSHELL=`pwd`/$ORADOEXECSHELL
FILENAME=$1
CheckPmon $SID
GetDBversion									#COR-047
GetORACLEversion								#COR-047
GetJobNumber RETURN
if [ $USERACTIVE = "NO" ]; then
	LogRecord "NOTE:  perfstat user account hasn't been created."
	FUNCTION=cruser
	spcuser
	FUNCTION=import
else
	LogRecord "NOTE:  perfstat user account has been created."
	#ClearfAndExit 1
fi
if [ $JOBNO != "NO" ]; then
	LogRecord "ERROR: statspack job, number $JOBNO is running."
	ClearfAndExit 1
fi
if [ "${FILENAME}_x" = "_x" ]; then
	if [ ! -d $ORATMP ]; then
		LogRecord "ERROR: import function needs a file name."
		ClearfAndExit 1
	fi
	cd $ORATMP
	$BIN/ls spuexp.dmp.* > $IMPFILELIST
	if [ ! -s $IMPFILELIST ]; then
		LogRecord "ERROR: import function needs a file name."
		ClearfAndExit 1
	fi
	LogRecord "NOTE:  import function needs a file name."
	N=`$USRBIN/wc -l $IMPFILELIST | $BIN/awk '{ print $1 }'`
	if [ $N -gt 1 ]; then
		if [ $BATCHMODE = "YES" ]; then
			LogRecord "NOTE:  any import files list couldn't be displayed."
			ClearfAndExit 1
		fi
		input="NO"
		while [ $input = "NO" ]
		do
			blankline
			$BIN/ls -l spuexp.dmp.* | $BIN/awk '{
			printf("%s      %3s %2s %s %9d\n",$9,$6,$7,$8,$5) }' > $IMPFILELIST
			$BIN/awk '{ printf("%10d.      %s\n",NR,$0) }' $IMPFILELIST
			$BIN/echo   "         Several import files found."
			$BIN/echo   ""
			$ECHOCMD -n "         Select file number ? : "
			read INPUT
			if [ "${INPUT}_x" != "_x" ]; then
				if [ ${INPUT} -le ${N} ] && [ ${INPUT} -ne 0 ]; then	# COR-038
					input="YES"
					FILENAME=`$BIN/awk '
					{	if ( NR == "'$INPUT'" )
							printf("%s\n",$1) }' $IMPFILELIST`
				fi
			else
				input="NO"
			fi
		done
		LogRecord "INFO:  the import file $FILENAME was chosen."
	else
		FILENAME=`$BIN/cat $IMPFILELIST`
		LogRecord "NOTE:  the import file $FILENAME was assumed."
	fi
	if [ -s $IMPFILELIST ]; then
		$BIN/rm $IMPFILELIST
	fi
else
	if [ ! -s $FILENAME ]; then
		if [ ! -s $EXECDIRECTORY/$FILENAME ]; then
			if [ ! -s $ORATMP/$FILENAME ]; then
				LogRecord "ERROR: file $FILENAME hasn't been found."
				ClearfAndExit 1
			else
				FILENAME=$ORATMP/$FILENAME
			fi
		else
			FILENAME=$EXECDIRECTORY/$FILENAME
		fi
	fi
fi
SetCONNstring perfstat								#COR-043
$ORABIN/imp userid=$CONN file=$FILENAME 1>$ORATMP/import.log 2>&1		#COR-043
if [ $? -ne 0 ]; then
	LogRecord "ERROR: the return status of imp command wasn't zero."
	LogRecord "WARN:  check $ORATMP/import.log."
	ClearfAndExit 1
fi
LogRecord "INFO:  import perfstat was successfully completed."
}

#
# Create PERFSTAT user. It is used to be preparing import.
#
spcuser () {
SPFILELIST="
spcusr.sql
dbmspool.sql
dbmsjob.sql
prvtpool.plb
"
cd $ORATMP
CheckPmon $SID
CheckUserAccount RETURN
if [ $USERACTIVE = "YES" ]; then
	LogRecord "ERROR: perfstat user account has already been created."
	ClearfAndExit 1
fi
for f in `$BIN/echo $SPFILELIST`
do
	if [ -s $ADMINDIR/${f} ]; then
		$BIN/cp $ADMINDIR/${f} ${f}
		if [ $? -ne 0 ]; then		# COR-038
			LogRecord "ERROR: ${f} copy failed."
			ClearfAndExit 1
		fi
	else
		LogRecord "ERROR: ${f} not found."
		ClearfAndExit 1
	fi
done
# COR-043 start.
if [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	# COR-051
	SetCONNstring system
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
	connect / as sysdba
	alter session set container = $PDBNAME;
	alter session set nls_language='AMERICAN';
	define perfstat_password='perfstat'
	define default_tablespace='$tools'
	define temporary_tablespace='temp'
	@spcusr
	grant connect,resource to perfstat;
	spool $SPOOLFILE
	show sqlcode;
	spool off
	EOF
else
	# COR-045 start.
	if [ $LEVEL = "8" ]; then
		SetCONNstring system
		$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
		connect / as sysdba
		@spdusr
		EOF
	fi
	# COR-045 end.
	SetCONNstring system
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
	connect / as sysdba
	alter session set nls_language='AMERICAN';
	define perfstat_password='perfstat'
	define default_tablespace='$tools'
	define temporary_tablespace='temp'
	@spcusr
	spool $SPOOLFILE
	show sqlcode;
	spool off
	EOF
fi
# COR-043 end.
$BIN/cat spcusr.lis >> $ORATMP/$FUNCTION.log
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
LogRecord "INFO:  perfstat user account was successfully created."
}

#
# Show ORACLE parameters for STATSPACK.
#
spparam () {
cd $ORATMP
CheckPmon $SID
CheckSysPasswd
GetDBversion									#COR-047
GetORACLEversion								#COR-047
# COR-004 start.
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
set LINESIZE 128
spool $SPOOLFILE
show parameter job_queue_processes;
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
JOBQUE=`$BIN/awk '{	if ( $1 == "job_queue_processes" ) {
				found="YES"
				value=$3
				exit
				}
			}
END {	printf("%d\n",value)	}' $SPOOLFILE`
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR=043
alter session set nls_language='AMERICAN';
set LINESIZE 128
spool $SPOOLFILE
select	name, value from v\$parameter
	where name = 'timed_statistics';
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
TIMESTAT=`$BIN/awk '{ if ( $1 == "timed_statistics" ) {
				found="YES"
				getline
				value=$1
				exit
				}
			}
END {	printf("%s\n",value)	}' $SPOOLFILE`
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
set LINESIZE 128
spool $SPOOLFILE
select	name, value from v\$parameter
	where name like 'undo%';
show sqlcode;
spool off
EOF
# COR-004 end.
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
UNDOPARAM=`$BIN/awk '{ if ( $1 == "undo_suppress_errors" ) {
				found="YES"
				getline
				value=$1
				exit
				}
			}
END {	printf("%s\n",value)	}' $SPOOLFILE`
TRUNC=` $BIN/grep "dbms_job.submit" $ADMINDIR/spauto.sql |
$BIN/sed "s/'//g" | $BIN/sed "s/,//g" |
$BIN/$nawk '{	for ( i = 1 ; i <= NF ; i++ ) {				#COR-011
			if ( i == 3 )
				field3=toupper($i)			#COR-028
			if ( i == 4 )
				if ( toupper($i) == field3 )		#COR-028
					result="OK"
				else
					result="NG"
		}
			
}
END { printf("%s\n",result) }'`
if [ $TRUNC = "OK" ]; then
	TRUNC=` $BIN/grep "dbms_job.submit" $ADMINDIR/spauto.sql |
	$BIN/awk -F, '{
		split($3,m,"/")
		split(m[2],n,",")
		truncm=n[1]
		o=substr($4,2,2)
		truncn=o
		exit
		}
	END	{	printf("%-4d  %s \n",truncm,truncn) }'`
else
	TRUNC="????  ??"
fi
$BIN/echo "
		PARAMETER                   VALUE
		--------------------------- --------
		job_queue_processes         $JOBQUE
		timed_statistics            $TIMESTAT
		undo_suppress_errors        $UNDOPARAM
		spauto.sql trunc function   $TRUNC
		--------------------------- --------
	" > $ORATMP/$FUNCTION.log
$BIN/cat $ORATMP/$FUNCTION.log
}

#
# Gather ORACLE information using SQL*PLUS.
#
spexsqls () {
if [ ! -d $SQLDIR ]; then
	$BIN/mkdir $SQLDIR 1>/dev/null 2>&1
fi
CheckPmon $SID
CheckSysPasswd
GetDBversion									#COR-046
GetORACLEversion								#COR-045
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 86
set pagesize 50000
col username             for a28
col account_status       for a18
col default_tablespace   for a18
col temporary_tablespace for a18
select username, account_status,
       default_tablespace, temporary_tablespace
       from dba_users
       order by username;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql510.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 104
set pagesize 50000
col tablespace_name for a18
col initial_extent  for 999,999,999,999
col next_extent     for 999,999,999,999
col max_extents     for 999,999,999,999
col pct_increase    for 999,999,999,999
col min_extlen      for 999,999,999,999
select tablespace_name, initial_extent, next_extent,
       max_extents, pct_increase, min_extlen
       from dba_tablespaces
       order by tablespace_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql521.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 50
set pagesize 50000
select tablespace_name, contents, status
       from dba_tablespaces
       order by tablespace_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql522.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 50
set pagesize 50000
select tablespace_name, extent_management, allocation_type
       from dba_tablespaces
       order by tablespace_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql523.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 96
set pagesize 50000
col file_name       for a64
col tablespace_name for a16
col autoextensible  for a14
select file_name, tablespace_name, autoextensible
       from dba_data_files
       order by file_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql524.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 96
set pagesize 50000
col file_name       for a64
col tablespace_name for a16
col autoextensible  for a14
select file_name, tablespace_name, autoextensible
       from dba_temp_files
       order by file_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql525.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 110
set pagesize 50000
col file_name for a64
col ts_name   for a14
col status    for a14
col enabled   for a14
select d.NAME file_name, t.NAME ts_name, d.status, d.enabled
       from v\$datafile d, v\$tablespace t
       where t.ts# = d.ts#
       order by file_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql526.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 76
set pagesize 50000
col username  for a34
col bytes     for 999,999,999,999,999
col max_bytes for 999,999,999,999,999
select username, bytes, max_bytes
       from dba_ts_quotas
       order by username;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql530.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 108
set pagesize 50000
col segment_name    for a36
col owner           for a12
col tablespace_name for a18
col extents         for 999,999,999,999
col blocks          for 999,999,999,999
select segment_name, owner, tablespace_name, extents, blocks
       from dba_segments
       order by segment_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql541.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 92
set pagesize 50000
col block_id  for 999999999999
col file_id   for 999999999999
col extent_id for 999999999999
col owner     for a34
col blocks    for 999999999999
select block_id, file_id, extent_id, owner, blocks
       from dba_extents
       order by owner, block_id;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql542.log
CheckSQLcode
printdot
SetCONNstring system							#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 80
set pagesize 50000
col tablespace_name for a30
col sum(blocks) for 999,999,999,999,999
select tablespace_name, count(*), sum(blocks)
       from dba_free_space
       group by tablespace_name
       order by tablespace_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql543.log
CheckSQLcode
printdot
SetCONNstring system
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set lines 90
set pagesize 50000
col name    for a44
col value   for a44
select name, value
       from v\$parameter
       order by name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql544.log
CheckSQLcode
printdot
SetCONNstring system							#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set lines 80
set pagesize 50000
show parameter
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql551.log
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	name from v\$controlfile;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql552.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	records_total, records_used
	from v\$controlfile_record_section
	where type = 'DATAFILE';
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql553.log
printdot
# COR-047 start.
if [ $LEVEL = "12" ] || [ $LEVEL = "18" ] || [ $LEVEL = "19" ]; then	#COR-051
	SetCONNstring system							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set container = $PDBNAME;
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select	name, log_mode from v\$database;
	show sqlcode;
	spool off
	EOF
else
	SetCONNstring system							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select	name, log_mode from v\$database;
	show sqlcode;
	spool off
	EOF
fi
# COR-047 end.
printdot
$BIN/cp $SPOOLFILE $SQLDIR/sql561.log
CheckSQLcode
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	group#, sequence#, members, status
	from v\$log;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql562.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 86
set pagesize 50000
col group#     for 9999999999
col status     for a8
col member     for a64
select group#, status, member from v\$logfile
       order by group#;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql563.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 96
set pagesize 50000
col segment_name    for a34
col tablespace_name for a34
col owner           for a6
col status          for a16
select segment_name, tablespace_name, owner, status
       from dba_rollback_segs
       order by segment_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql571.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 96
set pagesize 50000
col name    for a30
col extents for 999,999,999,999
col rssize  for 999,999,999,999
col xacts   for 999,999,999,999
col status  for a14
select	n.name, s.extents, s.rssize, s.xacts, s.status
	from v\$rollname n, v\$rollstat s
	where n.usn = s.usn;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql572.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	s.username, t.xidusn, n.name, t.ubafil, t.ubablk, t.used_ublk
	from v\$session s, v\$transaction t, v\$rollname n
	where s.saddr = t.ses_addr and t.xidusn = n.usn;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql573.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	n.usn, n.name, s.xacts, s.status
	from v\$rollname n, v\$rollstat s
	where n.name in (
		select	segment_name
			from dba_rollback_segs
			where tablespace_name = 'UNDOTBS1' )
	and n.usn = s.usn;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql574.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	to_char(begin_time, 'YY-MM-DD HH24:MI:SS') begin_time,
	to_char(end_time,   'YY-MM-DD HH24:MI:SS') end_time, undoblks
	from v\$undostat order by 1;
show sqlcode;
spool off
EOF
#$BIN/cp $SPOOLFILE $SQLDIR/sql575.log
#CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	table_name, index_name, monitoring, used
	from v\$object_usage;
show sqlcode;
spool off
EOF
#$BIN/cp $SPOOLFILE $SQLDIR/sql581.log
#CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set lines 104
set pagesize 50000
col index_name  for a34
col index_type  for a28
col uniqueness  for a14
col owner       for a14
col status      for a10
select index_name, index_type, uniqueness, owner, status
       from dba_indexes
       order by index_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql582.log
CheckSQLcode
printdot
#SetCONNstring system								#COR-043
#$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
#spool $SPOOLFILE
#select	o.object_name
#	from dba_objects o
#	where owner = 'PERFSTAT'
#	and o.object_id in (
#		select	i.obj#
#			from ????? i
#			where bitand ( i.property, 4 ) = 4 );
#show sqlcode;
#spool off
#EOF
$BIN/echo "T.B.D" > $SQLDIR/sql583.log
#CheckSQLcode
#printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set lines 128
set pagesize 50000
col index_name  for a34
col index_owner for a24
col table_name  for a34
col column_name for a24
select index_name, index_owner, table_name, substr(column_name,1,10) column_name
       from dba_ind_columns
       order by index_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql584.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 94
set pagesize 50000
col     profile       for a30
col     resource_name for a30
col     resource_type for a20
col     limit         for a10
select	profile, resource_name, resource_type, limit
	from dba_profiles
	order by resource_name;
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql591.log
CheckSQLcode
printdot
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
set wrap off
set lines 128
set pagesize 50000
col username      for a24
col profile       for a30
col resource_name for a34
col limit         for a10
select u.username, p.profile, p.resource_name, p.limit
       from dba_users u, dba_profiles p
       where p.profile = u.profile
       and p.resource_type ='KERNEL';
show sqlcode;
spool off
EOF
$BIN/cp $SPOOLFILE $SQLDIR/sql592.log
CheckSQLcode
blankline
}

#
# Display PERFSTAT user status.
#
spstatus () {
CheckPmon $SID
CheckSysPasswd
GetDBversion									#COR-047
GetORACLEversion								#COR-047
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool on
spool $SPOOLFILE
col username             for a18
col account_status       for a14
col default_tablespace   for a18
col temporary_tablespace for a20
select	username, account_status,
	default_tablespace, temporary_tablespace
	from dba_users
	where username = 'PERFSTAT';
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE > $ORATMP/$FUNCTION.log
CheckSQLcode
PERFSTAT=`$BIN/$nawk 'BEGIN { found="NO" }				#COR-011
	{
	if ( $1 == "PERFSTAT" ) {
		found="YES"
		username=$1
		account_status=$2
		default_tablespace=$3
		temporary_tablespace=$4
		exit
		}
	}
END {	if ( found == "YES" ) {
		printf("%s ",tolower(username))
		printf("%s ",tolower(account_status))
		printf("%s ",tolower(default_tablespace))
		printf("%s.\n",tolower(temporary_tablespace))
		}
	else
		printf("NO\n") }' $SPOOLFILE`
if [ "$PERFSTAT" = "NO" ]; then
	PERFSTAT="perfstat user is not created."
	USERACTIVE="NO"
	STATE=0
else
	USERACTIVE="YES"
	STATE=100
fi
if [ $USERACTIVE = "YES" ]; then
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool on
	spool $SPOOLFILE
	col log_user  for a10
	col job       for 9999999999
	col next_date for a9
	col next_sec  for a9
	col interval  for a26
	select	log_user, job, next_date, next_sec, interval
		from user_jobs;
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
	JOBS=`$BIN/$nawk '{						#COR-011
		if ( $1 == "PERFSTAT" ) {
			found="YES"
			log_user=$1
			job=$2
			next_date=$3
			next_sec=$4
			interval=$5
			exit
		}
	}
	END {	if ( found == "YES" ) {
			printf("%s ",tolower(log_user))
			printf("%d ",job)
			printf("%s ",next_date)
			printf("%s ",next_sec)
			printf("%s\n",interval)
			}
		else
			printf("NO\n") }' $SPOOLFILE`
	if [ "$JOBS" = "NO" ]; then
		JOBS="no any jobs running."
		JOBACTIVE="NO"
	else
		JOBACTIVE="YES"
		STATE=`$USRBIN/expr $STATE + 10`
	fi
fi
if [ $USERACTIVE = "YES" ]; then
	if [ -s $ORATMP/snap.log ]; then
		$BIN/rm $ORATMP/snap.log
	fi
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool on
	spool $SPOOLFILE
	select	snap_id,
		to_char(snap_time, 'dd Mon YYYY HH24:mi:ss') snap_time
		from stats\$snapshot, v\$database order by snap_id;
	show sqlcode;
	spool off
	EOF
	$BIN/cp $SPOOLFILE $ORATMP/snap.log
	CheckSQLcode
	N=`$BIN/awk '{
	if ( $1 == "SNAP_ID" ) {
		getline
		if ( $1 == "----------" ) {
 			found="YES"
			exit
			}
		}
	} END {	if ( found == "YES" )
			printf("1\n")
		else
			printf("0\n") }' $ORATMP/snap.log`
	if [ $N -eq 1 ]; then
		STATE=`$USRBIN/expr $STATE + 1`
	fi
else
	$BIN/echo "" > $ORATMP/snap.log
fi
if [ $BATCHMODE = "NO" ]; then
	blankline > $TEMPFILE
	$BIN/echo "	USER   : $PERFSTAT" >> $TEMPFILE
	if [ $USERACTIVE = "YES" ]; then
		$BIN/echo "	JOBS   : $JOBS" >> $TEMPFILE
		$BIN/echo "	SNAPS  :" >> $TEMPFILE
		if [ -s $ORATMP/snap.log ]; then
			$BIN/$nawk '{					#COR-011
			if (( $1 == "SNAP_ID" ) && ( prflag != "ON" )) {
				printf("\t%7s %s\n",tolower($1),tolower($2))
				getline
				printf("\t%7s %s\n",substr($1,1,7),$2)
				getline
				prflag="ON"
				}
			if ( prflag == "ON" ) {
				if ( $1 == "SQL>" )
					exit
				if (( $1 != "SNAP_ID" ) && ( substr($1,1,2) != "--" ) && ( NF != 0 ) && ( NF != 1 ))
					printf("\t%7d %s %s %s %s\n",$1,$2,$3,$4,$5)
				}
			}' $ORATMP/snap.log >> $TEMPFILE
		fi
	fi
	blankline >> $TEMPFILE
	$BIN/more $TEMPFILE
	$BIN/rm $TEMPFILE
else
	$BIN/cat $ORATMP/snap.log >> $ORATMP/$FUNCTION.log
	$BIN/rm $ORATMP/snap.log
	if [ -f $SPOOLFILE ]; then
		$BIN/rm $SPOOLFILE
	fi
	if [ -f $TEMPFILE ]; then
		$BIN/rm $TEMPFILE
	fi
	exit $STATE
fi
$BIN/cat $ORATMP/snap.log >> $ORATMP/$FUNCTION.log
$BIN/rm $ORATMP/snap.log
if [ -f $SPOOLFILE ]; then
	$BIN/rm $SPOOLFILE
fi
}

#
# Archive files.
#
sparchive () {
SEL=$1
CheckPmon $SID RETURN
if [ $RETVAL = "OK" ]; then
	# Check system account password.
	SetCONNstring system							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	spool off
	EOF
	#
	if [ $? -eq 0 ]; then		# COR-038
		GetJobNumber RETURN
		if [ $USERACTIVE = "NO" ]; then
			LogRecord "NOTE:  perfstat user account hasn't been created."
			#ClearfAndExit 1
		fi
		if [ $JOBNO != "NO" ]; then
			LogRecord "ERROR: job number $JOBNO has been running."
			ClearfAndExit 1
		fi
	fi
fi
if [ "${SEL}_x" = "_x" ]; then
	SEL=$SID
	LogRecord "INFO:  the target archive SID $SEL was assumed."
fi
cd $LOGDIR
$BIN/echo $SEL | $BIN/grep "_" >/dev/null 2>&1
if [ $? -ne 0 ]; then
	DIR=`$BIN/ls |
	$BIN/awk '{	n=split($1,array,"_")
			if (( n == 3 ) && ( array[3] == "'"$SEL"'" )) {
				check="YES"
				dirname=sprintf("%s_%s_%s\n",array[1],array[2],array[3])
				exit
				}
			}
		END {	if ( check == "YES" )
				printf("%s\n",dirname)
			else
				printf("NO\n") }'`
	if [ $DIR = "NO" ]; then
		LogRecord "ERROR: directory name checking was failed."
		ClearfAndExit 1
	fi
else
	if [ -f $TEMPFILE ]; then
		$BIN/rm $TEMPFILE 
	fi
	for i in `$BIN/ls | $BIN/grep orastats_${HOST}_${SEL}`
	do
		if [ -d $i ]; then
			$BIN/echo $i >> $TEMPFILE
		fi
	done
	DIRNUM=`$USRBIN/wc -l $TEMPFILE | $BIN/awk '{ printf("%d\n",$1) }'`
	if [ $DIRNUM -ne 1 ]; then
		LogRecord "ERROR: directory name checking was failed."
		ClearfAndExit 1
	fi
	DIR=`$BIN/cat $TEMPFILE`
fi
if [ -d $DIR ]; then
	TARFILE="${DIR}_`$BIN/date '+%y%m%d%H%M%S'`.tar"
	$BIN/rm $SPOOLFILE $TEMPFILE
	$BIN/tar -cvf $TARFILE $DIR
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: tar command failed."
		ClearfAndExit 1
	else
		if [ -x $BIN/gzip ]; then
			$BIN/gzip $TARFILE
			if [ $? -ne 0 ]; then
				LogRecord "ERROR: gzip command failed."
				ClearfAndExit 1
			fi
		# COR-019 start.
		else
			if [ -x $UCONTRIBIN/gzip ]; then
				$UCONTRIBIN/gzip $TARFILE
				if [ $? -ne 0 ]; then
					LogRecord "ERROR: gzip command failed."
					ClearfAndExit 1
				fi
			fi
		# COR-019 end.
		fi
	fi
	ROOTDIR=/
	TESTDIR=$EXECDIRECTORY
	FOUND=NO
	while [ "$ROOTDIR" != "$TESTDIR" ]
	do
		TESTDIR=`$USRBIN/dirname $TESTDIR`
		if [ $TESTDIR = $LOGDIR ]; then
			FOUND=YES
			ROOTDIR=/
		fi
	done
	if [ $FOUND = "YES" ]; then
		$BIN/echo "NOTE:  Although tar archive and compress were successfully completed,"
		$BIN/echo "NOTE:  the working directory $EXECDIRECTORY is below $LOGDIR."
		$BIN/echo "NOTE:  $DIR directory won't be cleared."
	else
		$BIN/rm -rf $DIR
		$BIN/echo "INFO:  tar archive and compress were successfully completed."
	fi
	ClearfAndExit
else
	LogRecord "ERROR: target archive directory not found."
	ClearfAndExit 1
fi
}

#
# Check ORACLE uid.
#
UidCheck () {
if [ $UIDCHECK = "YES" ]; then
	GID=`$BIN/awk -F= '{	if ( $1 == "inst_group" ) {
					found="YES"
					group=$2
					exit
				}
			}
	END {	if ( found == "YES" )
			printf("%s\n",group)
		else
			printf("NO\n") }' $ORAINST`
	if [ $GID = "NO" ]; then
		LogRecord "ERROR: ORACLE group not found in $ORAINST."
		ClearfAndExit 1
	fi
	OID=`$BIN/ls -l $ORACLE_HOME |
	$BIN/awk '{	if ( $4 == "'"$GID"'" ) {
				found="YES"
				uid=$3
				exit
				}
			}
	END {	if ( found == "YES" )
			printf("%s\n",uid)
		else
			printf("NO\n") }'`
	if [ $OID = "NO" ]; then
		LogRecord "ERROR: ORACLE uid not found in $ORACLE_HOME."
		ClearfAndExit 1
	fi
	ORACLEUID=`$BIN/awk -F: '{
		if ( $1 == "'"$OID"'" ) {
			found="YES"
			id=$3
			exit
			}
		}
		END {	if ( found == "YES" )
				printf("%d\n",id)
			else
				printf("NO\n")
		}' $PASSWDFILE`
	if [ $ORACLEUID = "NO" ]; then
		LogRecord "ERROR: ORACLE uid not found in $PASSWDFILE."
		ClearfAndExit 1
	fi
	USERID=`$USRBIN/id | $BIN/awk '{split($1,array,"=")
					split(array[2],id,"(")
					printf("%d\n",id[1]) }'`
	if [ $ORACLEUID -ne $USERID ]; then
		LogRecord "ERROR: $USERID isn't ORACLE uid."
		ClearfAndExit 1
	fi
fi
}

#
# Do sppurge.sql SQL.
#
DoSppurge () {
GetDBversion									# COR-047
GetORACLEversion								# COR-047
SetCONNstring perfstat								# COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					# COR-043
alter session set nls_language='AMERICAN';
define losnapid=$losnapid
define hisnapid=$hisnapid
@sppurge
$losnapid
$hisnapid
spool $SPOOLFILE
show sqlcode
spool off
EOF
CheckSQLcode
$BIN/cat sppurge.lis >> $ORATMP/$FUNCTION.log
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
$BIN/rm sppurge.lis
}

#
# purge function will run SQL*PLUS command sppurge.sql to clear snap(s).
#
sppurge () {
PURGELIMIT=24
SNAPS=$1
$BIN/cp $ADMINDIR/sppurge.sql $ORATMP/sppurge.sql
cd $ORATMP
if [ "${SNAPS}_x" = "_x" ]; then
	LogRecord "INFO:  purge function needs losnapid-hisnapid."
	if [ $BATCHMODE = "YES" ]; then
		ClearfAndExit 1
	else
		LogRecord "NOTE:  all snaps will be cleared."
		input="NO"
		while [ $input = "NO" ]
		do
			$ECHOCMD -n "       Are you sure ? : "
			read INPUT
			if [ "${INPUT}_x" != "_x" ]; then
				case $INPUT in
				Y|y)	input="YES"
					SNAPS=all		;;
				N|n)	input="YES"
					ClearfAndExit 1	;;
				*)				;;
				esac
			fi
		done
	fi
fi
CheckPmon $SID
GetJobNumber RETURN
if [ $USERACTIVE = "NO" ]; then
	LogRecord "ERROR: perfstat user account hasn't been created."
	ClearfAndExit 1
fi
if [ $JOBNO != "NO" ]; then
	LogRecord "ERROR: job number $JOBNO has been running."
	ClearfAndExit 1
fi
if [ $SNAPS != "all" ]; then
	$BIN/rm losnapid hisnapid snapdiff error 1>/dev/null 2>&1
	IDCHECK=`$BIN/echo $SNAPS |
	$BIN/$nawk 'BEGIN { error = "NO" }				#COR-011
	{	n = split($1,array,"-")
		if ( n == 2 ) {
			if ( array[1] > array[2] ) {
				printf("%s ",$0) >>"error"
				error="YES"
				}
			else {
				id="YES"
				printf("%d\n",array[1]) >"losnapid"
				printf("%d\n",array[2]) >"hisnapid"
				printf("%d\n",array[2]-array[1]) >"snapdiff"
				}
			}
		else {
			printf("%s ",$0) >>"error"
			error="YES"
			}
		}
	END {	if ( error == "NO" )
			printf("OK\n")
		else
			printf("NO\n")
	}'`
	if [ $IDCHECK = "NO" ]; then
		LogRecord "ERROR: sppurge argument missing. [ `$BIN/cat error` ]."
		$BIN/rm losnapid hisnapid snapdiff error 1>/dev/null 2>&1
		ClearfAndExit 1
	else
		losnapid=`$BIN/cat losnapid`
		LOSNAP=$losnapid
		hisnapid=`$BIN/cat hisnapid`
		HISNAP=$hisnapid
		snapdiff=`$BIN/cat snapdiff`
		SPDIFF=$snapdiff
	fi
	$BIN/rm losnapid hisnapid snapdiff error 1>/dev/null 2>&1
fi
SetCONNstring perfstat								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
select	db_name
	from stats\$database_instance;
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
DBNAME=`	$BIN/awk '{	if ( substr($1,1,7) == "DB_NAME" ) {
					getline
					getline
					name=substr($1,1,7)
					}
				}
END { printf("%s\n",name) }' $SPOOLFILE`
# COR-031 start.
if [ $OS = "CYGWIN" ]; then
	SetCONNstring perfstat							#COR-043
	$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF				#COR-043
	alter session set nls_language='AMERICAN';
	spool $SPOOLFILE
	select	instance_name
		from stats\$database_instance;
	show sqlcode;
	spool off
	EOF
	$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
	CheckSQLcode
	ORACLE_SID=`
	$BIN/awk '{	if ( $1 == "INSTANCE_NAME" ) {
				getline
				getline
				name=$1
				}
		}
	END { printf("%s\n",name) }' $SPOOLFILE`
fi
# COR-031 end.
# COR-005 start.
SetCONNstring perfstat								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
set LINESIZE 256
set trimspool on
spool $SPOOLFILE
set heading off
select	s.snap_id,
	d.instance_name,
	d.db_name,
	to_char(s.snap_time, 'dd Mon YYYY HH24:mi:ss')
		from stats\$snapshot s, stats\$database_instance d
		where s.dbid = d.dbid
			and s.instance_number = d.instance_number
			and s.startup_time = d.startup_time
			order by snap_id;
show sqlcode;
spool off
EOF
# COR-005 end.
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
# COR-018 start.
#if [ $OS = "CYGWIN" ]; then
#	ORACLE_SID=$ORADO
#fi
# COR-018 end.
$BIN/awk '{	if (( $2 == "'"$ORACLE_SID"'" ) && ( $3 == "'"$DBNAME"'" )) {
			if ( $1 != 0 )
				printf("%d\n",$1)
			if ( $1 == "SQL>" )
				exit
			}
}' $SPOOLFILE | $BIN/sort -n > $TEMPFILE
LOSNAPID=`$USRBIN/head -1 $TEMPFILE`
HISNAPID=`$USRBIN/tail -1 $TEMPFILE`
if [ "${LOSNAPID}_x" = "_x" ] || [ "${HISNAPID}_x" = "_x" ]; then
	LogRecord "NOTE:  no any snap records found."
	$BIN/rm $SPOOLFILE
	ClearfAndExit 1
fi
SNAPDIFF=`$USRBIN/expr $HISNAPID - $LOSNAPID`
ROWCOUNT=`$USRBIN/wc -l $TEMPFILE | $BIN/awk '{ printf("%d\n",$1) }'`
if [ $SNAPS != "all" ]; then
	losnapid=$LOSNAPID
	if [ $LOSNAP -eq $LOSNAPID ] || [ $LOSNAP -le $HISNAPID ] ; then
		losnapid=$LOSNAP
	else
		LogRecord "ERROR: sppurge argument missing [ $SNAPS ]."
		ClearfAndExit 1
	fi
	hisnapid=$HISNAPID
	if [ $HISNAP -eq $HISNAPID ] || [ $HISNAP -le $HISNAPID ]; then
		hisnapid=$HISNAP
	fi
	snapdiff=`$USRBIN/expr $hisnapid - $losnapid`
else
	losnapid=$LOSNAPID
	hisnapid=$HISNAPID
	snapdiff=$ROWCOUNT
fi
if [ $snapdiff -gt $PURGELIMIT ]; then
	end="NO"
	while [ $end = "NO" ]
	do
		DoSppurge
		losnapid=`$USRBIN/expr $losnapid + $PURGELIMIT`
		snapdiff=`$USRBIN/expr $snapdiff - $PURGELIMIT`
		if [ $snapdiff -lt $PURGELIMIT ]; then
			PURGELIMIT=$snapdiff
		fi
		if [ $snapdiff -eq 0 ]; then
			end="YES"
		fi
	done
else
	DoSppurge
fi
LogRecord "INFO:  statspack snap [ $SNAPS ] has been pureged."
}

#
# Change ORACLE parameters.
#
altersystem () {
p1=$1
p2=$2
SetCONNstring system								#COR-043
$ORABIN/sqlplus $CONN 1>$TEMPFILE 2>&1 <<-EOF					#COR-043
alter session set nls_language='AMERICAN';
spool $SPOOLFILE
alter system set $p1 = $p2;
show sqlcode;
spool off
EOF
$BIN/cat $SPOOLFILE >> $ORATMP/$FUNCTION.log
CheckSQLcode
LogRecord "INFO:  param $p1 has been changed to $p2."
}

#
# change system password.
#
spsetpwd () {
	ChgSysPasswd
}

#
# Run all orado.sh functions automatically.
#
spautoexec () {
#
MINUTES=$1	# Snap interval in minutes.
STARTTIME=$2	# Start time.
STOPTIME=$3	# Stop time.
#
BATCHMODE=YES; export BATCHMODE
RETRYCOUNT=5
RETRYTIMER=3
ERROR=NO
CheckPmon $SID
CheckSysPasswd
GetDBversion
GetORACLEversion
N=1
PARAMETERCHECKED=NO
while [ $PARAMETERCHECKED != "YES" ]
do
	$ORADOEXECSHELL param > $WORKFILE
	param=timed_statistics
	RESULT=`$BIN/awk '{
	if (( $1 == "timed_statistics" ) && ( $2 == "TRUE" )) {
		result="OK"
		exit
		}
	}
	END {	if ( result == "OK" )
			printf("%s\n",result)
		else
			printf("NG\n")
	}' $WORKFILE`
	if [ $RESULT != "OK" ]; then
		timed_statistics=NG
		altersystem $param true
	else
		timed_statistics=OK
	fi
	if [ $LEVEL != "9" ] && [ $LEVEL != "10" ] && [ $LEVEL != "11" ] && [ $LEVEL != "12" ] && [ $LEVEL != "18" ] && [ $LEVEL != "19" ]; then	#COR-027 COR-043 COR-051
		param=undo_suppress_errors
		RESULT=`$BIN/awk '{
		if ( $1 == "undo_suppress_errors" ) {
			if (( $2 == "TRUE" ) || ( NF != 2 )) {
				result="OK"
				exit
				}
			}
		}
		END {	if ( result == "OK" )
				printf("%s\n",result)
			else
				printf("NG\n")
		}' $WORKFILE`
		if [ $RESULT != "OK" ]; then
			undo_suppress_errors=NG
			altersystem $param true
		else
			undo_suppress_errors=OK
		fi
	else
		undo_suppress_errors=OK
	fi
	param=job_queue_processes
	RESULT=`$BIN/awk '{
	if (( $1 == "job_queue_processes" ) && ( $2 != 0 )) {
		result="OK"
		exit
		}
	}
	END {	if ( result == "OK" )
			printf("%s\n",result)
		else
			printf("NG\n")
	}' $WORKFILE`
	if [ $RESULT != "OK" ]; then
		job_queue_processes=NG
		altersystem $param $NUMBEROFJOBQ
	else
		job_queue_processes=OK
	fi
	RESULT=` $BIN/grep "dbms_job.submit" $ADMINDIR/spauto.sql |
	$BIN/sed "s/'//g" | $BIN/sed "s/,//g" |
	$BIN/$nawk '{	for ( i = 1 ; i <= NF ; i++ ) {			#COR-011
				if ( i == 3 )
					field3=tolower($i)
				if ( i == 4 )
					if ( tolower($i) == field3 )
						result="OK"
					else
						result="NG"
			}
			
	}
	END { printf("%s\n",result) }'`
	if [ $RESULT = "NG" ]; then
		LogRecord "ERROR: dbms_job.submit time parameters conflict."
		ClearfAndExit 1
	fi
	RESULT=`$BIN/awk '{
	if (( $1 == "spauto.sql" ) && ( $4 == "24" ) && ( $5 == "HH" )) {
		result="OK"
		exit
		}
	}
	END {	if ( result == "OK" )
			printf("%s\n",result)
		else
			printf("NG\n")
	}' $WORKFILE`
	if [ $RESULT = "NG" ]; then
		TRUNCM=` $BIN/grep "dbms_job.submit" $ADMINDIR/spauto.sql |
		$BIN/awk -F, '{
			split($3,m,"/")
			split(m[2],n,",")
			truncm=n[1]
			exit
			}
		END	{	printf("%d\n",truncm)	}'`
		if [ $TRUNCM != "24" ]; then
			truncm=NG
			$BIN/ed -s $ADMINDIR/spauto.sql 1>/dev/null 2>&1 <<-EOF
			/dbms_job.submit/
			s/$TRUNCM/24/g
			1,$ w $ADMINDIR/spauto.sql
			q
			EOF
			if [ $? -ne 0 ]; then
				LogRecord "ERROR: spauto.sql correction was failed."
				ClearfAndExit 1
			fi
		else
			truncm=OK
		fi
		TRUNCN=` $BIN/grep "dbms_job.submit" $ADMINDIR/spauto.sql |
		$BIN/awk -F, '{
			truncn=substr($4,2,2)
			exit
			}
		END	{	printf("%s\n",truncn) }'`
		if [ $TRUNCN != "HH" ]; then
			truncn=NG
			$BIN/ed -s $ADMINDIR/spauto.sql 1>/dev/null 2>&1 <<-EOF
			/dbms_job.submit/
			s/$TRUNCN/HH/g
			1,$ w $ADMINDIR/spauto.sql
			q
			EOF
			if [ $? -ne 0 ]; then
				LogRecord "ERROR: spauto.sql correction was failed."
				ClearfAndExit 1
			fi
			LogRecord "INFO:  spauto.sql script has been changed."
		else
			truncn=OK
		fi
	else
		truncm=OK
		truncn=OK
	fi
	if [ $job_queue_processes = "OK" ] && [ $timed_statistics = "OK" ] && [ $undo_suppress_errors = "OK" ] && [ $truncm = "OK" ] && [ $truncn = "OK" ]; then
		PARAMETERCHECKED=YES
	else
		N=`$USRBIN/expr $N + 1`
		if [ $N -gt $RETRYCOUNT ]; then
			LogRecord "ERROR: system parameter setting was faild."
			ClearfAndExit 1
		else
			$BIN/sleep $RETRYTIMER
		fi
	fi
done
SetTimeParameters $MINUTES $STARTTIME $STOPTIME
export INTERVAL COUNT ctime CTIME stime STIME etime ETIME SAMPLING
LogRecord "INFO:  sampling interval  : $INTERVAL"
LogRecord "INFO:  sampling count     : $COUNT"
LogRecord "INFO:  current time       : $ctime $CTIME"
LogRecord "INFO:  start and end time : $stime $STIME $etime $ETIME"
LogRecord "INFO:  sampling mode      : $SAMPLING"
if [ $ctime -gt $etime ]; then
	LogRecord "ERROR: the current time greater than the end time."
	ClearfAndExit 1
fi
$ORADOEXECSHELL status
case $? in
0)	LogRecord "INFO:  statspack can be setup and started."
	$ORADOEXECSHELL setup
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: setup failed."
		ClearfAndExit 1
	fi						;;
100)	LogRecord "INFO:  statspack can be started."
	$ORADOEXECSHELL clean
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: clean failed."
		ClearfAndExit 1
	fi
	$ORADOEXECSHELL setup
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: setup failed."
		ClearfAndExit 1
	fi						;;
101)	LogRecord "WARN:  any snaps have been kept."
	ClearfAndExit 1					;;
111)	LogRecord "ERROR: the job has been running."
	ClearfAndExit 1					;;
*)	LogRecord "WARN:  unknown statspack state."
	ClearfAndExit 1					;;
esac
MIN=$INTERVAL
if [ $stime -gt $ctime ]; then
	waitsec=`$USRBIN/expr $stime - $ctime`
	export waitsec
	LogRecord "INFO:  awaiting start $MIN [ min ] in $waitsec sec."
	$BIN/sleep $waitsec
else
	waitsec=0
fi
export waitsec
LogRecord "INFO:  snap job will be started with $MIN minutes interval."
$ORADOEXECSHELL start $MIN
if [ $? -ne 0 ]; then
	LogRecord "ERROR: start $MIN [ min ] failed."
	ClearfAndExit 1
fi
GetNextSec
if [ $NEXT_SEC = "NO" ]; then
	LogRecord "ERROR: statspack job couldn't be scheduled."
	ClearfAndExit 1
else
	$BIN/echo $NEXT_SEC > $ORATMP/run
fi

while [ -f $ORATMP/run ]
do
	LogRecord "INFO:  wait for the end time [ $ETIME ] to finish."
	if [ $SAMPLING = "INTHEDAY" ]; then
		REMAIN=`$USRBIN/expr $etime - $ctime`
	else
		REMAIN=`$BIN/echo $ctime | $BIN/awk '{
		todaysec = '"$ONEDAYSECONDS"' - $1
		nextdaysec = '"$etime"' - '"$ONEDAYSECONDS"'
		sec = todaysec + nextdaysec }
		END { printf("%d\n",sec) }'`
	fi
	REMAIN=`$USRBIN/expr $REMAIN - $waitsec`
	LogRecord "INFO:  remaining time is $REMAIN seconds."
	$BIN/sleep $REMAIN
	#
	BATCHMODE=NO; export BATCHMODE
	CURRENTTIME=`$ORADOEXECSHELL nextsec`
	BATCHMODE=YES
	#
	if [ $CURRENTTIME != "NO" ]; then
		S2I $CURRENTTIME
	else
		LogRecord "WARN:  statspack spauto job might have been terminated."
		if [ -f $ORATMP/run ]; then
			$BIN/rm $ORATMP/run
		else
			LogRecord "WARN:  the run file might have been cleared."
		fi
		TIMEVALUE=999999
	fi
	LogRecord "INFO:  NEXT_SEC will be compared to the end time."
	if [ -f $ORATMP/run ]; then
		$BIN/rm $ORATMP/run
	fi
	if [ $TIMEVALUE != "999999" ]; then
		$ORADOEXECSHELL stop
		if [ $? -ne 0 ]; then
			LogRecord "WARN:  job stop failed."
			ClearfAndExit 1
		fi
	fi
	$ORADOEXECSHELL report
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: report function failed."
		$BIN/cat $WORKFILE >> $ORATMP/$FUNCTION.log
		ClearfAndExit 1
	fi
	RESULTCOUNT=`$BIN/awk '{
	if (( $2 == "REPORT" ) && ( $5 == "result" ))
		n++
	}
	END { printf("%d\n",n) }' $TRACELOG`
	PASSEDCOUNT=`$BIN/awk '{
	if (( $2 == "REPORT" ) && ( $NF == "passed." ))
		n++
	}
	END { printf("%d\n",n) }' $TRACELOG`
	if [ $RESULTCOUNT -ne $PASSEDCOUNT ]; then
		LogRecord "ERROR: report function failed."
		ClearfAndExit 1
	fi
	$ORADOEXECSHELL export
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: export function failed."
		ClearfAndExit 1
	fi
	$ORADOEXECSHELL clean
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: clean function failed."
		ClearfAndExit 1
	fi
	LogRecord "INFO:  auto function was successfully completed."
	$ORADOEXECSHELL tar ${SID} > $LOGDIR/$FUNCTION.log		#COR-012
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: tar function failed."
		$BIN/cat $WORKFILE >> $ORATMP/$FUNCTION.log

		# COR-012 start.
		if [ -f $LOGDIR/$FUNCTION.log ]; then
			$BIN/cat $LOGDIR/$FUNCTION.log >> $ORATMP/$FUNCTION.log
			$BIN/rm $LOGDIR/$FUNCTION.log
		fi
		# COR-012 end.

		ClearfAndExit 1
	fi
	if [ -f $WORKFILE ]; then
		$BIN/rm $WORKFILE
	fi

	# COR-012 start.
	if [ -f $LOGDIR/$FUNCTION.log ]; then
		$BIN/rm $LOGDIR/$FUNCTION.log
	fi
	# COR-012 end.

	ClearfAndExit
done 2>/dev/null &
}

#
# Unpack orastats data into the orado directory.
#
spunpack () {
cd $LOGDIR
$BIN/ls  orastats_*_*.tar.gz 2>/dev/null > $IMPFILELIST
if [ ! -s $IMPFILELIST ]; then
	$BIN/echo "ERROR: no any unpack files."
	ClearfAndExit 1
fi
N=`$USRBIN/wc -l $IMPFILELIST | $BIN/awk '{ print $1 }'`
if [ $N -gt 1 ]; then
	if [ $BATCHMODE = "YES" ]; then
		$BIN/echo "NOTE:  any unpack files list couldn't be displayed."
		ClearfAndExit 1
	fi
	input="NO"
	while [ $input = "NO" ]
	do
		blankline
		$BIN/ls -l orastats_*_*.tar.gz | $BIN/awk '{
		printf("%s  %3s %2s %s %9d\n",$9,$6,$7,$8,$5) }' > $IMPFILELIST
		$BIN/awk '{ printf("%10d.  %s\n",NR,$0) }' $IMPFILELIST
		$ECHOCMD    "         Several unpack files found."	#COR-025
		$ECHOCMD    ""						#COR-025
		$ECHOCMD -n "         Select file number ? : "
		read INPUT
		if [ "${INPUT}_x" != "_x" ]; then
			if [ ${INPUT} -le ${N} ] && [ ${INPUT} != 0 ]; then
				input="YES"
				FILENAME=`$BIN/awk '
				{	if ( NR == "'$INPUT'" )
						printf("%s\n",$1) }' $IMPFILELIST`
			fi
		else
			input="NO"
		fi
	done
	$BIN/echo "INFO:  the unpack file $FILENAME was chosen."
else
	FILENAME=`$BIN/cat $IMPFILELIST`
	$BIN/echo "NOTE:  the unpack file $FILENAME was assumed."
fi
if [ -s $IMPFILELIST ]; then
	$BIN/rm $IMPFILELIST
fi
FILENAME=`$BIN/echo $FILENAME |
$BIN/awk '{	split($1,array,".")
		printf("%s.tar\n",array[1],array[2]) }'`
DIRECTORYNAME=`$BIN/echo $FILENAME |
$BIN/awk '{	split($1,array,".")
		split(array[1],dir,"_")
		printf("%s_%s_%s\n",dir[1],dir[2],dir[3]) }'`
if [ -d $DIRECTORYNAME ]; then
	$BIN/echo "ERROR: directory $DIRECTORYNAME already exists."
	ClearfAndExit 1
fi
# COR-019 start.
if [ -x $USRBIN/gunzip ]; then
	$USRBIN/gunzip -d ${FILENAME} 2>/dev/null
	if [ $? -ne 0 ]; then
		$BIN/echo "ERROR: gunzip command was failed."
		ClearfAndExit 1
	fi
else
	if [ -x $UCONTRIBIN/gunzip ]; then
		$UCONTRIBIN/gunzip -d ${FILENAME} 2>/dev/null
		if [ $? -ne 0 ]; then
			$BIN/echo "ERROR: gunzip command was failed."
			ClearfAndExit 1
		fi
	fi
fi
# COR-019 end.
$BIN/tar -xvf $FILENAME
if [ $? -ne 0 ]; then
	$BIN/echo "ERROR: tar -xvf command was failed."
	ClearfAndExit 1
fi
$BIN/echo "INFO:  spunpack was successfully completed."
}

#
# Get NEXT_SEC time.
#
spnextsec () {
GetDBversion									#COR-047
GetORACLEversion								#COR-047
GetNextSec
if [ $BATCHMODE = "NO" ]; then
	if [ "${NEXT_SEC}_x" = "_x" ]; then
		$BIN/echo "NOTIME"
	else
		$BIN/echo $NEXT_SEC
	fi
else
	if [ "${NEXT_SEC}_x" = "_x" ] || [ $NEXT_SEC = "NO" ] ; then
		exit 1
	else
		exit 0
	fi
fi
}

####################################
###   Main routine start here.   ###
####################################
#
# Set arguments.
#
# COR-011 start.
if [ -x /bin/uname ]; then
	OSNAME=`/bin/uname`
else
	if [ -x /usr/bin/uname ]; then
		OSNAME=`/usr/bin/uname`
	fi
fi
case $OSNAME in
Linux)		BIN=/bin
		ECHOCMD=$BIN/echo	;;
SunOS)		BIN=/usr/bin
		ECHOCMD=/usr/ucb/echo
		# COR-041 start.
		if [ ! -f $ECHOCMD ]; then
			ECHOCMD=/usr/bin/echo
		fi			;;
		# COR-041 end.
CYGWIN*)	BIN=/usr/bin
		ECHOCMD=$BIN/echo	;;
# COR-019 start.
HP-UX)		BIN=/usr/bin
		ECHOCMD=/usr/bin/echo
		UCONTRIBIN=/usr/contrib/bin	;;
# COR-019 end.
*)	echo "ERROR: operating system's name $OSNAME unknown."
	exit 1		;;
esac
USRBIN=/usr/bin
# COR-011 end.
#
ORADOSHELL=$0
numberofargs=$#
execshell=`$BIN/basename $ORADOSHELL`
FUNCTION=$1; ARG2=$2; ARG3=$3; ARG4=$4
if [ $numberofargs -eq 0 ]; then		# COR-038
	$BIN/echo "ERROR: argument missing."
	exit 1
fi
#
# determine the LOGDIR environment variable.
#
if [ "${LOGDIR}_x" = "_x" ]; then
	LOGDIR=/var/tmp
fi
#
# determine the SID environment variable.
#
if [ "${ORADO}_x" = "_x" ]; then
	if [ "${ORACLE_SID}_x" = "_x" ]; then
		SID=oracle							#COR-043
	else
		SID=$ORACLE_SID
	fi
else
	SID=$ORADO
fi
#
# COR-043 start.
# preset the LEVEL environment variable.
#
if [ "${PDBNAME}_x" = "_x" ];then
	LEVEL=10
else
	LEVEL=12
fi
# COR-043 end.
#
# set variables and parameters.
#
SetParams
#
# Make up the input/output directory for orado.sh.
#
if [ ! -d $ORATMP ] && [ $FUNCTION != "unpack" ]; then
	$BIN/mkdir $ORATMP
fi

WORKFILE=$ORATMP/workfile$$
#
# Clear the $FUNCTION.log file.
#
if [ -s $ORATMP/$FUNCTION.log ]; then
	$BIN/rm $ORATMP/$FUNCTION.log
fi
OS=`$BIN/uname -a |
$BIN/awk '{ printf("%s\n",substr($1,1,6)) }'`
if [ $OS = "CYGWIN" ]; then
	SPOOLFILE=on.lst$$
else
	SPOOLFILE=$ORATMP/on.lst$$
fi
#
# Get the working directory and the actual orado.sh path name.
#
EXECDIRECTORY=`$BIN/pwd`
whichcmd $execshell
RESULT=$COMMANDPATH
if [ $RESULT = "NOTFOUND" ]; then
	ORADOEXECSHELL=$ORADOSHELL
else
	ORADOEXECSHELL=$COMMANDPATH
fi
#
# Set SYSTEMPASSWORD environment variable of the first time running.
if [ "${SYSTEMPASSWORD}_x" = "_x" ]; then
	ChgSysPasswd
	ClearfAndExit
fi
#
# Set up ORACLE environment variables.
#
if [ $SPREPORT = "YES" ] && [ $OS != "CYGWIN" ]; then
	CheckAndSetOraEnv $SID
fi
#
# COR-016 start.
if [ $OS = "CYGWIN" ]; then
	if [ "${ORADO}_x" != "_x" ]; then
		ORABIN=`$BIN/echo $PATH | $BIN/sed 's/ /~/g' |		#COR-020
		$BIN/awk 'BEGIN { oradosid="'"$ORADO"'" }
		{
		n=split($1,array,":")
		for ( p = 1 ; p <= n ; p++ ) {
			fullpath=array[p]
			c=split(fullpath,path,"/")
			for ( q = 1 ; q <= c ; q++ ) {
				if ( path[q] == oradosid ) {
					printf("%s\n",fullpath)
					break
					}
				}
			}
		}'`
		if [ "${ORABIN}_x" = "_x" ]; then
			# COR-031 start.
			SQLPLUS=`which sqlplus`
			if [ $? -eq 0 ]; then
				RACHOME=`$BIN/dirname $SQLPLUS`
				RACHOME=`$BIN/dirname $RACHOME`
				ORABIN=$RACHOME/bin
				ADMINDIR=$RACHOME/rdbms/admin
			else
				$BIN/echo "ERROR: environment variable ORADO ${ORADO} or PATH missing."
				ClearfAndExit 1
			fi
			# COR-031 end.
		else
			# COR-020 start.
			ADMINDIR=`$BIN/dirname "$ORABIN" |
			$BIN/awk '{
			printf("%s/rdbms/admin\n",$0)
			}' | $BIN/sed 's/~/ /g'`
			# COR-020 end.
		fi
	else
		# COR-031 start.
		SQLPLUS=`which sqlplus`
		if [ $? -eq 0 ]; then
			RACHOME=`$BIN/dirname $SQLPLUS`
			RACHOME=`$BIN/dirname $RACHOME`
			ORABIN=$RACHOME/bin
			ADMINDIR=$RACHOME/rdbms/admin
		else
			$BIN/echo "ERROR: environment variable ORADO ${ORADO} missing."
			ClearfAndExit 1
		fi
		# COR-031 end.
	fi
else
	ORABIN=$ORACLE_HOME/bin
	ADMINDIR=$ORACLE_HOME/rdbms/admin
fi
# COR-016 end.
#
# Check and set the running mode.
#
if [ "${BATCHMODE}_x" = "_x" ]; then
	BATCHMODE=NO
else
	if [ $BATCHMODE != "YES" ] && [ $BATCHMODE != "NO" ]; then
		BATCHMODE=NO
	fi
fi
#
# Check ORACLE uid.
#
UidCheck
#
# Go to the individual function.
#
case $FUNCTION in
auto)		spautoexec ${ARG2} ${ARG3} ${ARG4}	;;
clean)		spclean					;;
export)		spexport				;;
import)		spimport   ${ARG2}			;;
list)		splists    ${ARG2} ${ARG3}		;;
nextsec)	spnextsec				;;
param)		spparam					;;
passwd)		spsetpwd				;;
purge)		sppurge    ${ARG2}			;;
report) 	spreport				;;
setup)		spsetup					;;
sql)		spexsqls				;;
start)		spstart    ${ARG2}			;;
status)		spstatus				;;
stop)		spstop					;;
tar)		sparchive  ${ARG2}			;;
unpack)		spunpack   ${ARG2}			;;
*)      LogRecord "ERROR: argument missing. [ $FUNCTION ]"
	ClearfAndExit 1					;;
esac
#
# clear work files and exit.
#
ClearfAndExit
#
# end script.
